@@ -1,36 +1,38 @@
-'''Pulsar provides several classes for writing clients. The first step
-is to subclass :class:`pulsar.ProtocolConsumer` which is needed
-for two reasons:
+'''
+This example illustrates how to write a simple TCP Echo server and client pair.
+The example is simple because the client and server protocols are symmetrical
+and therefore the :class:`EchoProtocol` will also be used as based class for
+:class:`EchoServerProtocol`.
+The code for this example is located in the :mod:`examples.echo.manage`
+module.
 
-* It encodes and sends the request to the remote server.
-* It listen (if needed) for incoming data from the remote server.
+Writing the Client
+=========================
 
-There are two methods which needs implementing:
+The protocol consumer
+~~~~~~~~~~~~~~~~~~~~~~~~
 
-* :meth:`pulsar.ProtocolConsumer.start_request` to kick start a request to a
-  remote server.
-* :meth:`pulsar.Protocol.data_received`, invoked by the
-  :class:`pulsar.Connection` when new data has been received from the
-  remote server.
+The first step is to subclass :class:`pulsar.ProtocolConsumer` to create the
+:class:`EchoProtocol` used by the client. The :class:`EchoProtocol` is needed
+for two reasons: 
 
-In this example we implement a very simple Echo server/client pair. Because
-this example has symmetric client and server protocols, the
-:class:`EchoProtocol` will also be used for the server.
+* It encodes and sends the request to the remote server via the
+  :meth:`EchoProtocol.start_request` method.
+* It listens for incoming data from the remote server via the
+  :meth:`EchoProtocol.data_received` method.
 
 
-Echo Protocol
-==================
+The client
+~~~~~~~~~~~~~~~
 
-.. autoclass:: EchoProtocol
-   :members:
-   :member-order: bysource
+Pulsar provides :ref:`additional classes <clients-api>` for writing
+clients handling multiple requests. Here we subclass the :class:`pulsar.Client`
+and implement the :class:`Echo.request` method. :class:`Echo` is the main
+client class, used in all interactions with the echo server::
 
-Echo Client
-==================
+    >>> client = Echo(('127,0,0,1', 8080))
+    >>> response = client.request(b'Hello!')
 
-.. autoclass:: Echo
-   :members:
-   :member-order: bysource
 
 Run The example
 ====================
@@ -49,8 +51,38 @@ than returning a :class:`pulsar.Deferred`.
 Check the :ref:`creating synchronous clients <tutorials-synchronous>` tutorial
 for further information.
 
-    >>> p.request('Hello')
-    'Hello'
+    >>> p.request(b'Hello')
+    b'Hello'
+    
+Implementation
+==================
+
+Echo Client Protocol
+~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: EchoProtocol
+   :members:
+   :member-order: bysource
+
+Echo Server Protocol
+~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: EchoServerProtocol
+   :members:
+   :member-order: bysource
+   
+Echo Client
+~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: Echo
+   :members:
+   :member-order: bysource
+   
+Echo Server
+~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. autofunction:: server
+
 '''
 try:
     import pulsar
@@ -63,7 +95,7 @@ from pulsar.apps.socket import SocketServer
 
 
 class EchoProtocol(pulsar.ProtocolConsumer):
-    '''A symmetric echo protocol consumer for client and servers.
+    '''A symmetric echo :class:`pulsar.ProtocolConsumer` for client and servers.
 The only difference between client and server is the implementation of the
 :meth:`response` method.'''
     separator = b'\r\n\r\n'
@@ -89,6 +121,8 @@ It simply search for the :attr:`separator` and, if found, it invokes the
             self.buffer += data
     
     def start_request(self):
+        '''Override :meth:`pulsar.Protocol.start_request` to write
+the message ended by the :attr:`separator` into the transport.'''
         self.transport.write(self.current_request.message + self.separator)
         
     def response(self, data):
@@ -99,9 +133,11 @@ message value, while servers sends the message back to the client.'''
         
 
 class EchoServerProtocol(EchoProtocol):
+    '''The :class:`pulsar.ProtocolConsumer` used by the echo :func:`server`.'''
     
     def response(self, data):
-        # write it back.
+        '''Override :meth:`EchoProtocol.response` method by writing the
+``data`` received back to the client.'''
         self.transport.write(data)
         # If we get a QUIT message, close the transport. Used by the test suite.
         if data[:-len(self.separator)] == b'QUIT':
@@ -109,7 +145,16 @@ class EchoServerProtocol(EchoProtocol):
     
     
 class Echo(pulsar.Client):
-    '''Echo client'''
+    '''Echo :class:`pulsar.Client`.
+    
+.. attribute:: full_response
+
+    Flag indicating if the :meth:`request` method should return the
+    :class:`EchoProtocol` handling the request (``True``) or a :class:`Deferred`
+    which will result in the server response message (``False``).
+    
+    Default: ``False``
+'''
     consumer_factory = EchoProtocol
 
     def __init__(self, address, full_response=False, **params):
@@ -131,12 +176,11 @@ which will be called once the server has sent back its response.'''
             return response.on_finished
             
         
-
 def server(description=None, **kwargs):
+    '''Create the :class:`pulsar.apps.socket.SocketServer` instance with
+:class:`EchoServerProtocol` as protocol factory.'''
     description = description or 'Echo Server'
-    return SocketServer(callable=EchoServerProtocol,
-                        description=description,
-                        **kwargs)
+    return SocketServer(EchoServerProtocol, description=description, **kwargs)
     
 
 if __name__ == '__main__':  #pragma nocover
