@@ -1,5 +1,8 @@
 import time
+import math
 from datetime import timedelta
+from random import random
+from functools import reduce
 
 from pulsar import get_request_loop, async_sleep
 from pulsar.apps import tasks
@@ -48,4 +51,23 @@ class CheckWorker(tasks.Job):
     def __call__(self, consumer):
         worker = consumer.worker
         backend = worker.app.backend
-        return {'tasks': list(backend.concurrent_tasks)}
\ No newline at end of file
+        return {'tasks': list(backend.concurrent_tasks)}
+    
+    
+class StandardDeviation(tasks.Job):
+    
+    def can_overlap(self, inputs=None, **kwargs):
+        return inputs is not None
+    
+    def __call__(self, consumer, inputs=None, sample=10, size=100):
+        if inputs is None:
+            for n in range(sample):
+                inputs = [random() for i in range(size)]
+                self.run_job(consumer, self.name, inputs=inputs)
+            return 'produced %s new tasks' % sample
+        else:
+            time.sleep(0.1)
+            v2 = reduce(lambda x,y: x+y, map(lambda x: x*x, inputs))/len(inputs)
+            return math.sqrt(v2)
+            
+        
\ No newline at end of file
