@@ -22,14 +22,14 @@ from pulsar.utils.httpurl import range
 
 class Graph(ws.WS):
     
-    def on_message(self, request, msg):
-        return json.dumps([(i,random()) for i in range(100)])
+    def on_message(self, websocket, msg):
+        websocket.write(json.dumps([(i,random()) for i in range(100)]))
+        
 
-    
 class Echo(ws.WS):
     
-    def on_message(self, request, msg):
-        return msg
+    def on_message(self, websocket, msg):
+        websocket.write(msg)
 
 
 class Site(wsgi.LazyWsgi):
@@ -45,7 +45,7 @@ class Site(wsgi.LazyWsgi):
         data = data % request.environ
         request.response.content_type = 'text/html'
         request.response.content = data
-        return request.response.start()
+        return request.response
 
 
 def server(**kwargs):
