@@ -9,8 +9,8 @@ The :class:`Configurator` is a mixin used as base class for both
 
 .. note::
 
-    An instance of an :class:`Application` is pickable and therefore can be sent
-    from actor to actor using the
+    An instance of an :class:`Application` is picklable and therefore can be
+    sent from actor to actor using the
     :ref:`actor message passing api <tutorials-messages>`.
 
 Configurator
@@ -68,18 +68,16 @@ Backend
 '''
 import os
 import sys
-import logging
 from hashlib import sha1
 from inspect import getfile
 
 import pulsar
-from pulsar import Actor, Monitor, get_actor, EventHandler
+from pulsar import Actor, Monitor, get_actor, EventHandler, async
 from pulsar.utils.structures import OrderedDict
 from pulsar.utils.pep import pickle
-from pulsar.utils.sockets import parse_connection_string, get_connection_string
+from pulsar.utils.internet import parse_connection_string, get_connection_string
 from pulsar.utils.log import LocalMixin
 from pulsar.utils.importer import import_module
-from pulsar.utils.security import gen_unique_id
 
 __all__ = ['Application',
            'MultiApp',
@@ -90,31 +88,29 @@ __all__ = ['Application',
     
     
 def get_application(name):
-    '''Fetch the :class:`Application` associated with *name* if available. This
-function returns a :ref:`coroutine <coroutine>` and therefore it
-must be used as an asynchronous component::
-
-    ...
-    app = yield pulsar.get_application('taskqueue')
-    ...
-
+    '''Fetch the :class:`Application` associated with ``name`` if available.
+    
+This function may return an :ref:`asynchronous component <coroutine>`.
+The application name is set during initialisation. Check the
+:attr:`Configurator.name` attribute for more information.
 '''
     actor = get_actor()
     if actor:
         if actor.is_arbiter():
-            yield _get_app(actor, name)
+            return _get_app(actor, name)
         else:
-            yield actor.send('arbiter', 'run', _get_app, name)
+            return actor.send('arbiter', 'run', _get_app, name)
         
 def _get_app(arbiter, name):
     monitor = arbiter.get_actor(name)
     if monitor:
         return monitor.params.app
-        
+
+@async()
 def monitor_start(self):
-    self.app.monitor_start(self)
+    yield self.app.monitor_start(self)
     if not self.cfg.workers:
-        self.app.worker_start(self)
+        yield self.app.worker_start(self)
     self.app.fire_event('start')
         
 def monitor_stop(self):
@@ -146,11 +142,6 @@ class Worker(Actor):
         '''The :class:`Application` served by this :class:`Worker`.'''
         return self.params.app
     
-    def io_poller(self):
-        '''Delegates the :meth:`pulsar.Actor.io_poller` method to the
-:meth:`Application.io_poller` method of the :attr:`app` attribute.'''
-        return self.app.io_poller(self)
-    
     def info(self):
         data = super(Worker, self).info()
         return self.app.worker_info(self, data)
@@ -329,7 +320,7 @@ The parameters overriding order is the following:
         # get the actor if available and override default cfg values with those
         # from the actor
         actor = get_actor()
-        if actor and actor.running:
+        if actor and actor.is_running():
             # actor available and running. unless argv is set, skip parsing
             if self.argv is None:
                 self.parsed_console = False
@@ -382,16 +373,17 @@ on :mod:`pulsar` concurrent framework.
 Applications can be of any sorts or forms and the library is shipped with
 several battery included examples in the :mod:`pulsar.apps` framework module.
 
-These are the most important facts about a pulsar :class:`Application`
+These are the most important facts about a pulsar :class:`Application`:
 
 * It derives from :class:`Configurator` so that it has all the functionalities
   to parse command line arguments and setup the :attr:`Configurator.cfg`.
-* Instances must be pickable. If non-pickable data needs to be add on an
+* Instances must be picklable. If non-picklable data needs to be add on an
   :class:`Application` instance, it must be stored on the
   :attr:`Application.local` dictionary.
-* When a new :class:`Application` is initialized,
+* Instances of an :class:`Application` are callable objects.
+* When an :class:`Application` is called for the first time,
   a new :class:`ApplicationMonitor` instance is added to the
-  :class:`Arbiter`, ready to perform its duties.
+  :class:`pulsar.Arbiter`, ready to perform its duties.
 
 :parameter callable: Initialise the :attr:`Application.callable` attribute.
 :parameter load_config: If ``False`` the :meth:`Configurator.load_config`
@@ -401,8 +393,8 @@ These are the most important facts about a pulsar :class:`Application`
 .. attribute:: callable
 
     Optional callable serving or configuring your application.
-    If provided, the callable must be pickable, therefore it is either
-    a function or a pickable object.
+    If provided, the callable must be picklable, therefore it is either
+    a function or a picklable object.
 
     Default ``None``
 """
@@ -468,31 +460,24 @@ and the :class:`Application` have the same interface.'''
     def event(self, name):
         return self.local.events.event(name)
 
-    def io_poller(self, worker):
-        '''Called by :meth:`Worker.io_poller` method during the initialization
-of the :class:`Worker` event loop. By default it does nothing so that
-the event loop chooses the most suitable IO poller.'''
-        return None
-
     def add_timeout(self, deadline, callback):
         self.arbiter.event_loop.add_timeout(deadline, callback)
     
     # WORKERS CALLBACKS
     def worker_start(self, worker):
-        '''Called by the :class:`Worker` :meth:`pulsar.Actor.on_start`
-:ref:`callback <actor-callbacks>` method.'''
+        '''Added to the ``start`` :ref:`worker hook <actor-hooks>`.'''
         pass
 
-    def worker_info(self, worker, data):
-        return data
+    def worker_info(self, worker, info):
+        '''Hook to add additional entries to the worker ``info`` dictionary.'''
+        return info
     
     def worker_stopping(self, worker):
-        '''Invoked when the *worker* fires the *stopping* event.'''
+        '''Added to the ``stopping`` :ref:`worker hook <actor-hooks>`.'''
         pass
     
     def worker_stop(self, worker):
-        '''Called by the :class:`Worker` :meth:`pulsar.Actor.on_stop`
-:ref:`callback <actor-callbacks>` method.'''
+        '''Added to the ``stop`` :ref:`worker hook <actor-hooks>`.'''
         pass
 
     # MONITOR CALLBACKS
@@ -508,6 +493,7 @@ The application is now in the arbiter but has not yet started.'''
         pass
 
     def monitor_info(self, monitor, data):
+        '''Hook to add additional entries to the monitor ``info`` dictionary.'''
         return data
     
     def monitor_stop(self, monitor):
@@ -519,7 +505,9 @@ The application is now in the arbiter but has not yet started.'''
         pass
     
     def start(self):
-        '''Start the application if it wasn't already started.'''
+        '''Start the :class:`pulsar.Arbiter` if it wasn't already started.
+Calling this method when the :class:`pulsar.Arbiter` is already running has
+no effect.'''
         arbiter = pulsar.arbiter()
         if arbiter and self.name in arbiter.registered:
             arbiter.start()
@@ -615,7 +603,7 @@ as the number of :class:`Application` required by this :class:`MultiApp`.
         
     def start(self):
         '''Use this method to start all applications at once.'''
-        apps = self.apps()
+        self.apps()
         arbiter = pulsar.arbiter()
         if arbiter:
             arbiter.start()
@@ -664,7 +652,19 @@ A :class:`Backend` is never initialised directly, the
     
 .. attribute:: connection_string
 
-    The connection string fro this backend.
+    The connection string fro this backend. It is always in the form::
+    
+        scheme:://address?param1=..&params2=... &...
+        
+    where ``address`` is usually a ``host:port`` string.
+    
+    A special connection string is::
+    
+        local://&param1=...&...
+        
+    which represents a backend implemented using pulsar itself. A local backend
+    is installed in the monitor managing the :class:`Application` which
+    createded it.
     
 .. attribute:: default_path
 
@@ -676,9 +676,12 @@ A :class:`Backend` is never initialised directly, the
         self.name = name or 'arbiter'
         self.connection_string = connection_string
         self.params = dict(self.setup(**params) or {})
-        value = '%s\n%s' % (self.__class__.__name__, self.connection_string)
-        self._id = sha1(value.encode('utf-8')).hexdigest()
+        self._id = sha1(str(self).encode('utf-8')).hexdigest()
         
+    def __repr__(self):
+        return '%s. %s' % (self.__class__.__name__, self.connection_string)
+    __str__ = __repr__
+    
     def setup(self, **params):
         return params
     
@@ -687,7 +690,10 @@ A :class:`Backend` is never initialised directly, the
         '''Identy for this backend, calculated from the class name and the
 :attr:`connection_string`. Therefore if two backend instances from
 the same class have the same connection string, than the :attr:`id`
-is the same.'''
+is the same.
+
+The :attr:`id`` is important when creating a new backend via the
+:meth:`make` method.'''
         return self._id
     
     @classmethod
@@ -697,7 +703,7 @@ which is of the form::
 
     scheme:://host?params
 
-For example, the local backend is::
+For example, a parameter-less local backend is::
 
     local://
     
@@ -705,7 +711,7 @@ A redis backend could be::
 
     redis://127.0.0.1:6379?db=1&password=bla
     
-:param backend: the connection string
+:param backend: the connection string, if not supplied ``local://`` is used.
 :param kwargs: additional key-valued parameters used by the :class:`Backend`
     during initialisation.
     '''
@@ -717,17 +723,23 @@ A redis backend could be::
         if scheme == 'local':
             if address[0] == '0.0.0.0':
                 address = ('',)
-            if name:
-                params['name'] = name
-        else:
-            kwargs['name'] = name
-        con_str = get_connection_string(scheme, address, params)
+        path = cls.path_from_scheme(scheme)
+        module = import_module(path)
+        bcls = getattr(module, cls.__name__)
+        con_str = bcls.get_connection_string(scheme, address, params, name)
+        kwargs['name'] = name
         params.update(kwargs)
         if 'timeout' in params:
             params['timeout'] = int(params['timeout'])
-        path = cls.path_from_scheme(scheme)
-        module = import_module(path)
-        return getattr(module, cls.__name__)(scheme, con_str, **params)
+        return bcls(scheme, con_str, **params)
+    
+    @classmethod
+    def get_connection_string(cls, scheme, address, params, name):
+        '''Create the connection string used during initialisation
+        of a new :attr:`Backend`.'''
+        if name:
+            params['name'] = name
+        return get_connection_string(scheme, address, params)
     
     @classmethod
     def path_from_scheme(cls, scheme):
