@@ -15,7 +15,7 @@ from collections import namedtuple
 from base64 import b64encode, b64decode
 
 import pulsar
-from pulsar import create_transport, is_failure
+from pulsar import is_failure
 from pulsar.utils.pep import native_str, is_string, to_bytes
 from pulsar.utils.structures import mapping_iterator
 from pulsar.utils.websocket import FrameParser, SUPPORTED_VERSIONS
@@ -299,7 +299,7 @@ Initialised by a call to the :class:`HttpClient.request` method.
                 raise HTTPError(self.url, self.status_code,
                                 self.content_string(), self.headers, None)
             else:
-                raise URLError(self.on_finished.result.trace[1])
+                raise URLError(self.on_finished.result.error)
     
     def get_origin_req_host(self):
         response = self.history[-1] if self.history else self
