@@ -1,5 +1,6 @@
 '''The :mod:`pulsar.apps.pubsub` implements a middleware
-handler for the Publish/Subscribe pattern.
+handler for the Publish/Subscribe pattern. The middleware can be used to
+synchronise pulsar actors across processes and machines.
 
 .. epigraph::
 
@@ -23,6 +24,9 @@ When using this middleware, one starts by creating a :class:`PubSub` handler::
 The ``backend`` parameter is needed in order to select the backend
 to use. If not supplied, the default ``local://`` backend is used.
 
+A backend handler is a picklable instance and therefore it can be passed to
+different process domains.
+
 
 PubSub handler
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -75,9 +79,6 @@ class PubSub(object):
 
     Optional callable which encode the messages before they are published.
     
-.. attribute:: clients
-
-    Set of all clients for this :class:`PubSub` handler.
 '''
     def __init__(self, backend=None, encoder=None, **params):
         be = PubSubBackend.make(backend=backend, **params)
@@ -86,6 +87,7 @@ class PubSub(object):
     
     @property
     def clients(self):
+        '''Set of all clients for this :class:`PubSub` handler.'''
         return self.backend.clients
     
     @property
@@ -116,13 +118,11 @@ via the :attr:`encoder` callable (if available).'''
         return self.backend.publish(channel, message)
     
     def subscribe(self, *channels):
-        '''Subscribe to the server which publish messages. Must be
-implemented by subclasses.'''
+        '''Invoke the :meth:`PubSubBackend.subscribe` method.'''
         return self.backend.subscribe(*channels)
     
     def unsubscribe(self, *channels):
-        '''Un-subscribe to the server which publish messages. Must be
-implemented by subclasses.'''
+        '''Invoke the :meth:`PubSubBackend.unsubscribe` method.'''
         return self.backend.unsubscribe(*channels)
     
     def close(self):
@@ -142,10 +142,6 @@ class PubSubBackend(pulsar.Backend):
         '''The set of clients for this :class:`PubSub` handler.'''
         return set()
     
-    @local_property
-    def lock(self):
-        return Lock()
-    
     @classmethod
     def path_from_scheme(cls, scheme):
         return 'pulsar.apps.pubsub.%s' % scheme
@@ -162,18 +158,25 @@ from the publisher, the :meth:`broadcast` method will notify all
         self.clients.discard(client)
         
     def publish(self, channel, message):
-        '''Publish a ``message`` into ``channel``. Must be implemented
-by subclasses.'''
+        '''Publish a ``message`` into ``channel``.
+        
+Must be implemented by subclasses.'''
         raise NotImplementedError
     
     def subscribe(self, *channels):
-        '''Subscribe to the server which publish messages. Must be
-implemented by subclasses.'''
+        '''Subscribe to the server which publish messages.
+        
+A series of one or more ``channels`` to subscribe to must be passed to this
+method which must be implemented by subclasses.'''
         raise NotImplementedError
     
     def unsubscribe(self, *channels):
-        '''Un-subscribe to the server which publish messages. Must be
-implemented by subclasses.'''
+        '''Un-subscribe from the server which publish messages.
+        
+An optional series of ``channels`` can be passed. If no channels are passed,
+it unsubscribed from all channels.
+
+Must be implemented by subclasses.'''
         raise NotImplementedError
         
     def broadcast(self, channel, message):
@@ -185,7 +188,7 @@ implemented by subclasses.'''
         for client in clients:
             try:
                 client(channel, message)
-            except Exception as e:
+            except Exception:
                 LOGGER.exception('Exception while processing pub/sub client. '
                                  'Removing it.')
                 remove.add(client)
