@@ -1,5 +1,5 @@
 '''
-The :class:`TaskBackend` is at the hart of the
+The :class:`TaskBackend` is at the heart of the
 :ref:`task queue application <apps-taskqueue>`. It exposes
 all the functionalities for running new tasks, scheduling periodic tasks
 and retrieving task information. Pulsar ships with two backends, one which uses
@@ -7,9 +7,17 @@ pulsar internals and store tasks in the arbiter domain and another which stores
 tasks in redis_.
 
 
-Implementing a Task Backend
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-When creating a new :class:`TaskBackend` there are five methods which must
+Overview
+===============
+
+The backend is created by the :class:`pulsar.apps.tasks.TaskQueue`
+as soon as it starts. It is then passed to all task queue workers
+which, in turns, invoke the :class:`TaskBackend.start` method
+to start pulling tasks form the distributed task queue.
+
+Implementation
+~~~~~~~~~~~~~~~~~
+When creating a new :class:`TaskBackend` there are six methods which must
 be implemented:
 
 * The :meth:`TaskBackend.put_task` method, invoked when putting a new
@@ -22,24 +30,8 @@ be implemented:
   or updating a :class:`Task`.
 * The :meth:`TaskBackend.delete_tasks` method, invoked when deleting
   a bunch of :class:`Task`.
-
-.. _apps-taskqueue-task:
-
-Task
-~~~~~~~~~~~~~
-
-.. autoclass:: Task
-   :members:
-   :member-order: bysource
-      
-   
-TaskBackend
-~~~~~~~~~~~~~
-
-.. autoclass:: TaskBackend
-   :members:
-   :member-order: bysource
-
+* The :meth:`TaskBackend.flush` method, invoked flushing a backend (remove
+  all tasks and clear the task queue).
 
 .. _task-state:
 
@@ -69,7 +61,45 @@ The set of states for which a :class:`Task` has run:
 
 The set of states for which a :class:`Task` has finished:
 ``REVOKED``, ``FAILURE`` and ``SUCCESS``
+
+.. _tasks-pubsub:
+
+Task status broadcasting
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+A :class:`TaskBackend` broadcast :class:`Task` state into three different
+channels via the :attr:`TaskBackend.pubsub` handler.
+
+The pubsub handler is constructed using the same
+:attr:`pulsar.apps.Backend.connection_string` and
+:attr:`pulsar.apps.Backend.name` as the the :class:`TaskBackend`. 
+
+API
+=========
+
+.. _apps-taskqueue-task:
+
+Task
+~~~~~~~~~~~~~
+
+.. autoclass:: Task
+   :members:
+   :member-order: bysource
+      
    
+TaskBackend
+~~~~~~~~~~~~~
+
+.. autoclass:: TaskBackend
+   :members:
+   :member-order: bysource
+   
+TaskConsumer
+~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: TaskConsumer
+   :members:
+   :member-order: bysource
    
 Scheduler Entry
 ~~~~~~~~~~~~~~~~~~~
@@ -78,17 +108,28 @@ Scheduler Entry
    :members:
    :member-order: bysource
    
+
+Local Backend
+==================
+
+.. automodule:: pulsar.apps.tasks.backends.local
+
+Redis Backend
+==================
+
+.. automodule:: pulsar.apps.tasks.backends.redis
+
+
 .. _redis: http://redis.io/
 '''
 import sys
 import logging
-from collections import deque
 from datetime import datetime, timedelta
-from functools import partial
+from threading import Lock
 
-from pulsar import async, EMPTY_TUPLE, EMPTY_DICT, get_actor, log_failure,\
-                maybe_failure, is_failure, PulsarException, Backend, Deferred
-from pulsar.utils.pep import itervalues, iteritems
+from pulsar import (maybe_async, EMPTY_TUPLE, EMPTY_DICT, Failure,
+                    PulsarException, Backend, Deferred, coroutine_return)
+from pulsar.utils.pep import itervalues
 from pulsar.apps.tasks.models import JobRegistry
 from pulsar.apps.tasks import states, create_task_id
 from pulsar.apps import pubsub
@@ -135,21 +176,25 @@ class TaskTimeout(PulsarException):
 class TaskConsumer(object):
     '''A context manager for consuming tasks.
 
+Instances of this consumer are created by the :class:`TaskBackend` when
+a task is executed.
+
 .. attribute:: task_id
 
     the :attr:`Task.id` being consumed.
 
 .. attribute:: job
 
-    the :class:`Job` which generated the :attr:`task`.
+    the :ref:`Job <apps-taskqueue-job>` which generated the :attr:`task`.
 
 .. attribute:: worker
 
-    the :class:`pulsar.apps.Worker` running the process.
+    the :class:`pulsar.Actor` running the task worker.
     
 .. attribute:: backend
 
-    give access to the :class:`TaskBackend`.
+    Access to the :class:`TaskBackend`. This is useful when creating
+    tasks from within a :ref:`job callable <job-callable>`.
 '''
     def __init__(self, backend, worker, task_id, job):
         self.backend = backend
@@ -201,10 +246,11 @@ class Task(object):
     of tasks within other tasks.
 '''
     stack_trace = None
-    def __init__(self, id, name=None, time_executed=None,
-                 expiry=None, args=None, kwargs=None, 
-                 status=None, from_task=None, result=None, **params):
+    def __init__(self, id, overlap_id='', name=None, time_executed=None,
+                 expiry=None, args=None, kwargs=None, status=None,
+                 from_task=None, result=None, **params):
         self.id = id
+        self.overlap_id = overlap_id
         self.name = name
         self.time_executed = time_executed
         self.from_task = from_task
@@ -259,7 +305,7 @@ class PubSubClient(pubsub.Client):
         
     def __call__(self, channel, message):
         if channel == self.task_done:
-            self._be.task_done_callback(message)
+            maybe_async(self._be.task_done_callback(message))
             
         
 class TaskBackend(Backend):
@@ -293,19 +339,30 @@ It also schedules the run of periodic tasks if enabled to do so.
     Passed by the task-queue application
     :ref:`max requests setting <setting-max_requests>`.
     
+.. attribute:: poll_timeout
+
+    The (asynchronous) timeout for polling tasks from the task queue. It is
+    always a positive number and it can be specified via the backend
+    connection string::
+    
+        local://?poll_timeout=3
+        
+    There shouldn't be any reason to modify the default value.
+    
+    Default: ``2``.
+    
 .. attribute:: processed
 
-    The number of tasks processed by the worker.
+    The number of tasks processed (so far) by the worker running this backend.
+    This value is important in connection with the :attr:`max_tasks` attribute.
     
-.. attribute:: pubsub
-
-    A :class:`pulsar.apps.pubsub.PubSub` handler for tasks signals.
 '''
     def setup(self, task_paths=None, schedule_periodic=False, backlog=1,
-              max_tasks=0, **params):
+              max_tasks=0, poll_timeout=None, **params):
         self.task_paths = task_paths
         self.backlog = backlog
         self.max_tasks = max_tasks
+        self.poll_timeout = max(poll_timeout or 0, 2)
         self.processed = 0
         self.local.schedule_periodic = schedule_periodic
         self.next_run = datetime.now()
@@ -320,27 +377,36 @@ It also schedules the run of periodic tasks if enabled to do so.
         return self.local.schedule_periodic
     
     @local_property
-    def callbacks(self):
-        '''Dictionary of :class:`pulsar.Deferred` with keys given by
-:attr:`Task.id` which this instance is waiting on. The :class:`pulsar.Deferred`
-are called back once the task has finished. This dictionary is used by
-the :meth:`wait_for_task` method.'''
-        return {}
+    def lock(self):
+        return Lock()
     
     @local_property
     def pubsub(self):
-        '''A :class:`pulsar.apps.pubsub.PubSub` handler which notify tasks
-execution status.'''
+        '''A :class:`pulsar.apps.pubsub.PubSub` handler which notifies
+and listen tasks execution status. There are three channels:
+
+* ``<name>_task_created`` published when a new task is created.
+* ``<name>_task_start`` published when the task queue starts executing a task.
+* ``<name>_task_done`` published when a task is done.
+
+All three messages are composed by the task id only. Here ``<name>`` is replaced
+by the :attr:`pulsar.Backend.name` attribute of this task backend.
+
+Check the :ref:`task broadcasting documentation <tasks-pubsub>` for more
+information.
+'''
         p = pubsub.PubSub(backend=self.connection_string, name=self.name)
         p.add_client(PubSubClient(self))
-        c=  self.channel
+        c = self.channel
         p.subscribe(c('task_created'), c('task_start'), c('task_done'))
         return p
         
     @local_property
     def concurrent_tasks(self):
-        '''The set of tasks which are being concurrently executed by this
-:class:`TaskBackend`.'''
+        '''Concurrent set of task ids.
+        
+        The task with id in this set are currentlty being executed
+        by the task queue worker running this :class:`TaskBackend`..'''
         return set()
     
     @property
@@ -366,35 +432,150 @@ execution status.'''
     
     def run_job(self, jobname, targs=None, tkwargs=None, **meta_params):
         '''Create a new :ref:`task <apps-taskqueue-task>` which may or
-may not be queued. This method returns a :ref:`coroutine <coroutine>`.
-If *jobname* is not a valid :attr:`pulsar.apps.tasks.models.Job.name`,
+may not be queued. This method returns a :class:`pulsar.Deferred` which
+results in the :attr:`Task.id` created.
+If ``jobname`` is not a valid :attr:`pulsar.apps.tasks.models.Job.name`,
 a ``TaskNotAvailable`` exception occurs.
 
-:parameter jobname: the name of a :class:`Job` registered
-    with the :class:`TaskQueue` application.
+:parameter jobname: the name of a :class:`pulsar.apps.tasks.models.Job`
+    registered with the :class:`pulsar.apps.tasks.TaskQueue` application.
 :parameter targs: optional tuple used for the positional arguments in the
     task callable.
 :parameter tkwargs: optional dictionary used for the key-valued arguments
     in the task callable.
 :parameter meta_params: Additional parameters to be passed to the :class:`Task`
     constructor (not its callable function).
-:return: a :ref:`coroutine <coroutine>` resulting in a :attr:`Task.id`
+:return: a :class:`pulsar.Deferred` resulting in a :attr:`Task.id`
     on success.'''
-        return self._run_job(jobname, targs, tkwargs, meta_params)\
-                   .add_errback(log_failure)
+        c = self.create_task(jobname, targs, tkwargs, **meta_params)
+        return maybe_async(c, get_result=False).add_callback(self.put_task)
         
-    def create_task_id(self, job, args, kwargs):
-        '''Create a :attr:`Task.id` from *job*, positional arguments *args*
-and key-valued arguments *kwargs*.'''
-        return job.make_task_id(args, kwargs)
+    def wait_for_task(self, task_id, timeout=None):
+        '''Asynchronously wait for a task with ``task_id`` to have finished
+its execution. It returns a :class:`pulsar.Deferred`.'''
+        # make sure we are subscribed to the task_done channel
+        def _():
+            self.pubsub
+            task = yield self.get_task(task_id)
+            if task:
+                if task.done(): # task done, simply return it
+                    when_done = self.pop_callback(task.id)
+                    if when_done:
+                        when_done.callback(task)
+                    yield task
+                else:
+                    yield self.get_callback(task_id)
+        return maybe_async(_(), timeout=timeout, get_result=False)
+     
+    ############################################################################
+    ##    START/CLOSE METHODS FOR TASK WORKERS
+    ############################################################################
+    def start(self, worker):
+        '''invoked by the task queue ``worker`` when it starts.
         
+        Here, the ``worker`` creates its thread pool via
+        :meth:`pulsar.Actor.create_thread_pool` and register the
+        :meth:`may_pool_task` callback in its event loop.'''
+        worker.create_thread_pool()
+        self.local.task_poller = worker.event_loop.call_soon(
+                                    self.may_pool_task, worker)
+        worker.logger.debug('started polling tasks')
+        
+    def close(self, worker):
+        '''Close this :class:`TaskBackend`. Invoked by the
+:class:`pulsar.apps.Worker` when is stopping.'''
+        if self.local.task_poller:
+            self.local.task_poller.cancel()
+            worker.logger.debug('stopped polling tasks')
+    
+    ############################################################################
+    ##    ABSTRACT METHODS
+    ############################################################################
+    def put_task(self, task_id):
+        '''Put the ``task_id`` into the queue.
+
+:parameter task_id: the task id.
+:return: an :ref:`asynchronous component <tutorial-coroutine>` which results in
+    the ``task_id`` added.
+    
+**Must be implemented by subclasses.**'''
+        raise NotImplementedError
+    
+    def num_tasks(self):
+        '''Retrieve the number of tasks in the task queue.'''
+        raise NotImplementedError
+    
+    def get_task(self, task_id=None, when_done=False):
+        '''Retrieve a :class:`Task` from a ``task_id``. Must be implemented
+by subclasses.
+
+:param task_id: the :attr:`Task.id` of the task to retrieve.
+:param when_done: if ``True`` return only when the task is in a ready state.
+:return: a :class:`Task` or ``None``.
+
+**Must be implemented by subclasses.**
+'''
+        raise NotImplementedError
+    
+    def get_tasks(self, **filters):
+        '''Retrieve a group of :class:`Task` from the backend.
+        
+        **Must be implemented by subclasses.**'''
+        raise NotImplementedError
+    
+    def save_task(self, task_id, **params):
+        '''Create or update a :class:`Task` with ``task_id`` and key-valued
+parameters ``params``.
+
+**Must be implemented by subclasses.**'''        
+        raise NotImplementedError
+    
+    def delete_tasks(self, task_ids=None):
+        '''Delete a group of task. Must be implemented by subclasses.
+        
+        **Must be implemented by subclasses.**'''
+        raise NotImplementedError
+    
+    def flush(self):
+        '''Flush task backend by removing all tasks and clearing the queue.
+        
+        **Must be implemented by subclasses.**'''
+        raise NotImplementedError
+    
+    ############################################################################
+    ##    PRIVATE METHODS
+    ############################################################################
+    def tick(self, now=None):
+        '''Run a tick, that is one iteration of the scheduler. This
+method only works when :attr:`schedule_periodic` is ``True`` and
+the arbiter context.
+
+Executes all due tasks and calculate the time in seconds to wait before
+running a new :meth:`tick`. For testing purposes a :class:`datetime.datetime`
+value ``now`` can be passed.'''
+        if not self.schedule_periodic: 
+            return
+        remaining_times = []
+        try:
+            for entry in itervalues(self.entries):
+                is_due, next_time_to_run = entry.is_due(now=now)
+                if is_due:
+                    self.run_job(entry.name)
+                if next_time_to_run:
+                    remaining_times.append(next_time_to_run)
+        except Exception:
+            LOGGER.exception('Unhandled error in task backend')
+        self.next_run = now or datetime.now()
+        if remaining_times:
+            self.next_run += timedelta(seconds = min(remaining_times))
+            
     def create_task(self, jobname, targs=None, tkwargs=None, expiry=None,
                     **params):
         '''Create a new :class:`Task` from ``jobname``, positional arguments
 ``targs``, key-valued arguments ``tkwargs`` and :class:`Task` meta parameters
 ``params``. This method can be called by any process domain.
         
-:param jobname: the name of job which create the task.
+:param jobname: the name of the :class:`Job` which create the task.
 :param targs: task positional arguments (a ``tuple`` or ``None``).
 :param tkwargs: task key-valued arguments (a ``dict`` or ``None``).
 :return: a :ref:`coroutine <coroutine>` resulting in a :attr:`Task.id`
@@ -405,14 +586,19 @@ and key-valued arguments *kwargs*.'''
             job = self.registry[jobname]
             targs = targs or EMPTY_TUPLE
             tkwargs = tkwargs or EMPTY_DICT
-            task_id = self.create_task_id(job, targs, tkwargs)
-            task = yield self.get_task(task_id)
+            task_id, overlap_id = job.generate_task_ids(targs, tkwargs)
+            task = None
+            if overlap_id:
+                tasks = yield self.get_tasks(overlap_id=overlap_id)
+                # Tasks with overlap id already available
+                for task in tasks:
+                    if task.done():
+                        yield self.save_task(task.id, overlap_id='')
+                        task = None
+                    else:
+                        break
             if task:
-                # the task with id is already available
-                if task.done():
-                    task = yield self.handle_task_done(task)
-            if task:
-                LOGGER.debug('Task %s already requested, abort.', task)
+                LOGGER.debug('Task %s cannot run.', task)
                 yield None
             else:
                 if self.entries and job.name in self.entries:
@@ -423,8 +609,8 @@ and key-valued arguments *kwargs*.'''
                 elif job.timeout:
                     expiry = get_datetime(job.timeout, time_executed)
                 LOGGER.debug('Queue new task %s (%s).', job.name, task_id)
-                yield self.save_task(task_id, name=job.name,
-                                     time_executed=time_executed,
+                yield self.save_task(task_id, overlap_id=overlap_id,
+                                     name=job.name, time_executed=time_executed,
                                      expiry=expiry, args=targs, kwargs=tkwargs,
                                      status=states.PENDING, **params)
                 pubsub.publish(self.channel('task_created'), task_id)
@@ -481,147 +667,35 @@ and key-valued arguments *kwargs*.'''
             return (next_entry.name, max(next_time, 0))
         else:
             return (jobnames, None)
-    
-    def handle_task_done(self, task):
-        new_id = create_task_id()
-        data = task.tojson()
-        data.pop('id')
-        yield self.delete_tasks([task.id])
-        yield self.save_task(new_id, **data)
-        yield None
         
-    def wait_for_task(self, task_id):
-        '''Asynchronously wait for a task with ``task_id`` to have finished
-its execution. It returns an `asynchronous component <tutorials-coroutine>`_'''
-        # make sure we are subscribed to the task_done channel
-        self.pubsub
-        task = yield self.get_task(task_id)
-        if task:
-            if task.done(): # task done, simply return it
-                when_done = self.callbacks.pop(task.id, None)
-                yield when_done.callback(task) if when_done else task
-            else:
-                when_done = self.callbacks.get(task.id)
-                if not when_done:
-                    # No deferred, create one and check again
-                    self.callbacks[task.id] = when_done = Deferred()
-                    yield self.wait_for_task(task_id)
-                else:
-                    # deferred already available, return it
-                    yield when_done
-            
-    def tick(self, now=None):
-        '''Run a tick, that is one iteration of the scheduler. This
-method only works when :attr:`schedule_periodic` is ``True`` and
-the arbiter context.
-
-Executes all due tasks and calculate the time in seconds to wait before
-running a new :meth:`tick`. For testing purposes a :class:`datetime.datetime`
-value ``now`` can be passed.'''
-        if not self.schedule_periodic: 
-            return
-        remaining_times = []
-        try:
-            for entry in itervalues(self.entries):
-                is_due, next_time_to_run = entry.is_due(now=now)
-                if is_due:
-                    self.run_job(entry.name)
-                if next_time_to_run:
-                    remaining_times.append(next_time_to_run)
-        except Exception:
-            LOGGER.exception('Unhandled error in task backend')
-        self.next_run = now or datetime.now()
-        if remaining_times:
-            self.next_run += timedelta(seconds = min(remaining_times))
-     
-    ############################################################################
-    ##    START/CLOSE METHODS FOR TASK WORKERS
-    ############################################################################
-    def start(self, worker):
-        '''Start this :class:`TaskBackend`. Invoked by the
-:class:`pulsar.apps.Worker` which is ready to consume tasks.'''
-        worker.create_thread_pool()
-        self.local.task_poller = worker.event_loop.call_soon(
-                                    self.may_pool_task, worker)
-        LOGGER.debug('%s started polling tasks', worker)
-        
-    def close(self, worker):
-        '''Close this :class:`TaskBackend`. Invoked by the
-:class:`pulsar.apps.Worker` when is stopping.'''
-        if self.local.task_poller:
-            self.local.task_poller.cancel()
-            LOGGER.debug('%s stopped polling tasks', worker)
-    
-    ############################################################################
-    ##    ABSTRACT METHODS
-    ############################################################################
-    def put_task(self, task_id):
-        '''Put the ``task_id`` into the queue. Must be implemented
-by subclasses.'''
-        raise NotImplementedError
-    
-    def num_tasks(self):
-        '''Retrieve the number of tasks in the task queue.'''
-        raise NotImplementedError
-    
-    def get_task(self, task_id=None, when_done=False, timeout=1):
-        '''Retrieve a :class:`Task` from a ``task_id``. Must be implemented
-by subclasses.
-
-:param task_id: the :attr:`Task.id` of the task to retrieve.
-:param when_done: if ``True`` return only when the task is in a ready state.
-:param timeout: timeout to use when polling a task from the distributed queue.
-:return: a :class:`Task` or ``None``.
-'''
-        raise NotImplementedError
-    
-    def get_tasks(self, **filters):
-        '''Retrieve a group of :class:`Task` from the backend.'''
-        raise NotImplementedError
-    
-    def save_task(self, task_id, **params):
-        '''Create or update a :class:`Task` with ``task_id`` and key-valued
-parameters ``params``. Must be implemented by subclasses.'''        
-        raise NotImplementedError
-    
-    def delete_tasks(self, task_ids=None):
-        '''Delete a group of task. Must be implemented by subclasses.'''
-        raise NotImplementedError
-    
-    ############################################################################
-    ##    PRIVATE METHODS
-    ############################################################################
-    @async()
     def may_pool_task(self, worker):
-        '''Called at every loop in the worker IO loop, it pool a new task
-if possible and add it to the queue of tasks consumed by the worker
-CPU-bound thread.'''
+        '''Called in the ``worker`` event loop.
+        
+        It pools a new task if possible, and add it to the queue of
+        tasks consumed by the ``worker`` CPU-bound thread.'''
         next_time = 0
-        while worker.running:
+        if worker.is_running():
             thread_pool = worker.thread_pool
             if not thread_pool:
-                LOGGER.warning('No thread pool, cannot poll tasks.')
+                worker.logger.warning('No thread pool, cannot poll tasks.')
             elif self.num_concurrent_tasks < self.backlog:
                 if self.max_tasks and self.processed >= self.max_tasks:
                     if not self.num_concurrent_tasks:
                         worker.logger.warning('Processed %s tasks. Restarting.')
                         worker.stop()
-                        return
+                        coroutine_return()
                 else:
                     task = yield self.get_task()
                     if task:    # Got a new task
                         self.processed += 1
                         self.concurrent_tasks.add(task.id)
-                        thread_pool.apply_async(self._execute_task,
-                                                (worker, task))
+                        thread_pool.apply(self._execute_task, worker, task)
             else:
-                LOGGER.info('%s concurrent requests. Cannot poll.',
-                            self.num_concurrent_tasks)
-                next_time = 2*self.num_concurrent_tasks/5
-                break
+                worker.logger.info('%s concurrent requests. Cannot poll.',
+                                   self.num_concurrent_tasks)
+                next_time = 1
         worker.event_loop.call_later(next_time, self.may_pool_task, worker)
-            
-    @async(max_errors=None)
+    
     def _execute_task(self, worker, task):
         #Asynchronous execution of a Task. This method is called
         #on a separate thread of execution from the worker event loop thread.
@@ -641,7 +715,7 @@ CPU-bound thread.'''
                 if task.expiry and time_ended > task.expiry:
                     raise TaskTimeout
                 else:
-                    LOGGER.info('starting task %s', task)
+                    worker.logger.info('starting task %s', task)
                     yield self.save_task(task_id, status=states.STARTED,
                                          time_started=time_ended)
                     pubsub.publish(self.channel('task_start'), task_id)
@@ -650,12 +724,12 @@ CPU-bound thread.'''
             else:
                 consumer = None
         except TaskTimeout:
-            LOGGER.debug('Task %s timed-out', task)
+            worker.logger.info('Task %s timed-out', task)
             status = states.REVOKED
-        except:
-            result = maybe_failure(sys.exc_info())
-        if is_failure(result):
-            result.log()
+        except Exception:
+            result = Failure(sys.exc_info())
+        if isinstance(result, Failure):
+            result.log(msg='Failure in task %s' % task, log=worker.logger)
             status = states.FAILURE
             result = str(result)
         elif not status:
@@ -663,7 +737,7 @@ CPU-bound thread.'''
         if consumer:
             yield self.save_task(task_id, time_ended=time_ended,
                                  status=status, result=result)
-            LOGGER.debug('Finished task %s', task)
+            worker.logger.info('Finished task %s', task)
             # PUBLISH task_done
             pubsub.publish(self.channel('task_done'), task.id)
         self.concurrent_tasks.discard(task.id)
@@ -687,23 +761,36 @@ CPU-bound thread.'''
             raise ValueError('Schedule %s is not a timedelta' % s)
         return Schedule(s, anchor)
     
-    @async(get_result=False)
-    def _run_job(self, jobname, targs, tkwargs, meta_params):
-        # Create a new task and put it in the task queue
-        task_id = yield self.create_task(jobname, targs, tkwargs, **meta_params)
-        if task_id:
-            self.put_task(task_id)
-            yield task_id
-    
-    @async()
     def task_done_callback(self, task_id):
-        '''Got a new message from redis pubsub task_done channel.
-This is the write method required by all pubsub clients.'''
+        '''Got a task_id from the ``<name>_task_done`` channel.
+        
+Check if a ``callback`` is available in the :attr:`callbacks` dictionary. If
+so fire the callback with the ``task`` instance corresponsding to the input
+``task_id``.
+
+If a callback is not available, it must have been fired already.'''
         task = yield self.get_task(task_id)
         if task:
-            when_done = self.callbacks.pop(task.id, None)
+            when_done = self.pop_callback(task.id)
             if when_done:
                 when_done.callback(task)
+                
+    def pop_callback(self, task_id):
+        with self.lock:
+            return self.callbacks.pop(task_id, None)
+        
+    def get_callback(self, task_id):
+        with self.lock:
+            callbacks = self.callbacks
+            when_done = callbacks.get(task_id)
+            if not when_done:
+                # No deferred, create one and check again
+                callbacks[task_id] = when_done = Deferred()
+            return when_done
+    
+    @local_property
+    def callbacks(self):
+        return {}            
             
     
 class Schedule(object):
@@ -781,7 +868,7 @@ class SchedulerEntry(object):
     def anchor(self):
         return self.schedule.anchor
 
-    def next(self, now = None):
+    def next(self, now=None):
         """Returns a new instance of the same class, but with
         its date and count fields updated. Function called by :class:`Scheduler`
         when the ``this`` is due to run."""
@@ -790,5 +877,5 @@ class SchedulerEntry(object):
         self.total_run_count += 1
         return self
 
-    def is_due(self, now = None):
+    def is_due(self, now=None):
         return self.schedule.is_due(self.scheduled_last_run_at, now=now)
\ No newline at end of file
