@@ -1,7 +1,7 @@
-'''The :class:`TestSuite` is a testing framework for
-asynchronous/synchronous applications and for running tests in parallel
+'''The :class:`TestSuite` is a testing framework for both synchronous and
+asynchronous applications and for running tests in parallel
 on multiple threads or processes.
-It is used for testing pulsar itself but it can be used
+It is used for testing pulsar but it can be used
 as a test suite for any other library.
 
 Requirements
@@ -81,33 +81,38 @@ An example test case::
 Loading Tests
 =================
 
-Loading test cases is accomplished via the :class:`loader.TestLoader` class. In
-this context we refer to an ``object`` as a ``module`` (including a
-directory module) or a ``class``.
+The loading of test cases is controlled by the ``modules`` parameter when
+initialising the :class:`TestSuite`::
 
-.. note::
+    from pulsar.apps import TestSuite
 
-    Test modules are specified when initialising a :class:`TestSuite`, by
-    passing the ``modules`` tuple/list of python dotted paths. If not defined
-    the default is to load test cases from the ``tests`` module.
+    if __name__ == '__main__':
+        TestSuite(modules=('tests',
+                          ('examples','tests'))).start()
+                           
+The :class:`TestSuite` loads tests via the :class:`loader.TestLoader` class.
+
+In the context of explaining the rules for loading tests, we refer to
+an ``object`` as
+
+* a ``module`` (including a directory module)
+* a python ``class``.
     
+with this in mind:
 
-These are the rules for loading tests:
-
-* Directories that aren't packages are not inspected. This means that if a directory
-  does not have a ``__init__.py`` file it won't be inspected.
+* Directories that aren't packages are not inspected. This means that if a
+  directory does not have a ``__init__.py`` file, it won't be inspected.
 * Any class that is a ``unittest.TestCase`` subclass is collected.
-* If an object starts with ``_`` or ``.`` it won't be collected,
-  nor will any objects it contains.
-* If an object defines a ``__test__`` attribute that does not evaluate to
-  ``True``, that object will not be collected, nor will any objects it contains.
+* If an ``object`` starts with ``_`` or ``.`` it won't be collected,
+  nor will any ``objects`` it contains.
+* If an ``object`` defines a ``__test__`` attribute that does not evaluate to
+  ``True``, that object will not be collected, nor will any ``objects``
+  it contains.
   
-The paths which will be inspected are defined by the **modules** initialisation
-parameter for the :class:`TestSuite` class.
-The **modules** parameter is a tuple or list of entries where each single entry
-follow the following rules:
+The ``modules`` parameter is a tuple or list of entries where each single
+``entry`` follows the rules:
 
-* If the entry is a string, it indicates the **dotted path** relative to the
+* If the ``entry`` is a string, it indicates the **dotted path** relative to the
   **root** directory (the directory of the script running the tests). For
   example::
   
@@ -151,7 +156,8 @@ Options
 ==================
 
 All standard :ref:`settings <settings>` can be applied to the test application.
-In addition, the following options are testsuite-specific:
+In addition, the following options are
+:ref:`test-suite specific <setting-section-test>`:
 
 .. _apps-test-sequential:
 
@@ -160,7 +166,8 @@ sequential
 By default, test functions within a :class:`unittest.TestCase`
 are run in asynchronous fashion. This means that several test functions
 may be executed at once depending on their return values.
-By specifying the ``--sequential`` command line option, the :class:`TestSuite`
+By specifying the :ref:`--sequential <setting-sequential>` command line option,
+the :class:`TestSuite`
 forces tests to be run in a sequential model, one after the other::
 
     python runtests.py --sequential
@@ -177,7 +184,7 @@ in a sequential way, you can use the :func:`sequential` decorator::
         
 list labels
 ~~~~~~~~~~~~~~~~~~~
-By passing the ``-l`` or ``--list-labels`` flag to the command line, the
+By passing the ``-l`` or :ref:`--list-labels <setting-list_labels>` flag to the command line, the
 full list of test labels available is displayed::
 
     python runtests.py -l
@@ -187,7 +194,7 @@ test timeout
 ~~~~~~~~~~~~~~~~~~~
 When running asynchronous tests, it can be useful to set a cap on how
 long a test function can wait for results. This is what the
-``--test-timeout`` command line flag does::
+:ref:`--test-timeout <setting-test_timeout>` command line flag does::
 
     python runtests.py --test-timeout 10
     
@@ -228,14 +235,28 @@ Profile
 API
 =========
 
+.. _test-suite:
+
+Test Suite
+~~~~~~~~~~~~~~~~~~~~~~~
+
 .. autoclass:: TestSuite
    :members:
    :member-order: bysource
 
+
+Test Loader
+~~~~~~~~~~~~~~~~~~~~~~~
+
 .. automodule:: pulsar.apps.test.loader
 
+
+Plugins
+~~~~~~~~~~~~~~~~~~~~~~~
+
 .. automodule:: pulsar.apps.test.result
 
+
 .. module:: pulsar.apps.test
 
 
@@ -248,42 +269,33 @@ Test Plugin
    
 .. _unittest2: http://pypi.python.org/pypi/unittest2
 .. _mock: http://pypi.python.org/pypi/mock
+
+Utilities
+~~~~~~~~~~~~~~~
+
+.. automodule:: pulsar.apps.test.utils
+    :members:
 '''
-import logging
-import time
+import sys
 
 import pulsar
 from pulsar.apps import tasks
 from pulsar.utils import events
-from pulsar.utils.pep import ispy26, ispy33
 from pulsar.utils.log import local_property
+from pulsar.utils.config import section_docs
+from pulsar.utils.pep import default_timer
 
-if ispy26: # pragma nocover
-    try:
-        import unittest2 as unittest
-    except ImportError:
-        print('To run tests in python 2.6 you need to install\
- the unittest2 package')
-        exit(0)
-else:
-    import unittest
-    
-if ispy33: 
-    from unittest import mock
-else: # pragma nocover
-    try:
-        import mock
-    except ImportError:
-        print('To run tests you need to install the mock package')
-        exit(0)
-
-from .result import *
 from .case import *
+from .result import *
 from .plugins.base import *
 from .loader import *
 from .utils import *
 from .wsgi import *
 
+section_docs['Test'] = '''
+This section covers configuration parameters used by the
+:ref:`Asynchronous/Parallel test suite <apps-test>`.'''
+
 def dont_run_with_thread(obj):
     '''Decorator for disabling test cases when the test suite runs in
 threading, rather than processing, mode.'''
@@ -329,6 +341,14 @@ class TestLabels(TestOption):
 To see available labels use the -l option."""
 
 
+class TestExcludeLabels(TestOption):
+    name = "exclude_labels"
+    flags = ['-e', '--exclude-labels']
+    nargs = '*'
+    desc = 'Exclude a group o labels from running.'
+    validator = pulsar.validate_list
+
+
 class TestSize(TestOption):
     name = 'size'
     flags = ['--size']
@@ -354,6 +374,20 @@ class TestSequential(TestOption):
     validator = pulsar.validate_bool
     desc = """Run test functions sequentially. Don't run them asynchronously."""
     
+    
+class TestShowLeaks(TestOption):
+    name = "show_leaks"
+    flags = ['--show-leaks']
+    action = 'store_true'
+    default = False
+    validator = pulsar.validate_bool
+    desc = """Shows memory leaks.
+    
+    Run the garbadge collector before a process-based actor dies and shows
+    the memory leak report."""
+    
+
+pyver = '%s.%s' % (sys.version_info[:2])
 
 class TestSuite(tasks.TaskQueue):
     '''An asynchronous test suite which works like a task queue where each task
@@ -384,21 +418,19 @@ is a group of tests specified in a test class.
     cfg = pulsar.Config(apps=('tasks', 'test'),
                         loglevel='none',
                         task_paths=['pulsar.apps.test.case'],
-                        plugins=(),
-                        logconfig={
-                                   'loggers': {
-                                   'pulsar.test': {
-                                        'handlers': ['console_message'],
-                                        'level': logging.WARNING}
-                                    }
-                        })
+                        plugins=())
     @local_property
     def runner(self):
         '''Instance of :class:`TestRunner` driving the test case
 configuration and plugins.'''
+        if unittest is None:
+            raise ImportError('python %s requires unittest2 library for pulsar '
+                              'test suite application' % pyver)
+        if mock is None:
+            raise ImportError('python %s requires mock library for pulsar '
+                              'test suite application' % pyver)
         result_class = getattr(self, 'result_class', None)
-        r = unittest.TextTestRunner()
-        stream = r.stream
+        stream = get_stream(self.cfg)
         runner = TestRunner(self.cfg.plugins, stream, result_class)
         abort_message = runner.configure(self.cfg)
         if abort_message:
@@ -444,18 +476,27 @@ configuration and plugins.'''
         super(TestSuite, self).monitor_start(monitor)
         loader = self.local.loader
         tags = self.cfg.labels
+        exclude_tags = self.cfg.exclude_labels
+        if self.cfg.show_leaks:
+            self.cfg.set('when_exit', show_leaks)
+            arbiter = pulsar.arbiter()
+            arbiter.cfg.set('when_exit', show_leaks)
         try:
-            self.local.tests = tests = list(loader.testclasses(tags))
+            self.local.tests = tests = []
+            self.runner.on_start()
+            for tag, testcls in loader.testclasses(tags, exclude_tags):
+                suite = self.runner.loadTestsFromTestCase(testcls)
+                if suite and suite._tests:
+                    tests.append((tag, testcls))
             self._time_start = None
             if tests:
                 self.logger.info('loaded %s test classes', len(tests))
-                self.runner.on_start()
                 events.fire('tests', self, tests=tests)
                 monitor.cfg.set('workers', min(self.cfg.workers, len(tests)))
-                self._time_start = time.time()
+                self._time_start = default_timer()
                 for tag, testcls in self.local.tests:
                     self.backend.run('test', testcls, tag)
-                monitor.event_loop.call_every(self._check_queue)
+                monitor.event_loop.call_repeatedly(1, self._check_queue)
             else:
                 raise ExitTest('Could not find any tests.')
         except ExitTest as e:
@@ -475,19 +516,20 @@ configuration and plugins.'''
             cfg.settings.update(plugin.config.settings)
         return cfg
     
-    @pulsar.async()
     def _check_queue(self):
         runner=  self.runner
         tests = yield self.backend.get_tasks(status=tasks.READY_STATES)
         if len(tests) == len(self.local.tests):
-            time_taken = time.time() - self._time_start
+            self.logger.info('All tests have finished.')
+            time_taken = default_timer() - self._time_start
             for task in tests:
                 runner.add(task.result)
             runner.on_end()
             runner.printSummary(time_taken)
             # Shut down the arbiter
             if runner.result.errors or runner.result.failures:
-                exit_code = 1
+                exit_code = 2
             else:
                 exit_code = 0
-            raise pulsar.HaltServer(exit_code=exit_code)
\ No newline at end of file
+            raise pulsar.HaltServer(exit_code=exit_code)
+        
\ No newline at end of file
