@@ -1,4 +1,7 @@
 '''
+:class:`Plugin` is the interface used by the test suite to add functionalities.
+Some plugins, such as the :class:`TestRunner` are used by default.
+
 Plugin
 ~~~~~~~~~~~~~~~~~~~
 
@@ -17,13 +20,12 @@ Test Runner
    
 '''
 import traceback
-import unittest
-import logging
 from inspect import istraceback
 from copy import deepcopy
 
 from pulsar.utils.structures import AttributeDictionary
 
+from .case import unittest, LOGGER
 from .utils import TestFunction
 
 __all__ = ['Plugin',
@@ -34,7 +36,6 @@ __all__ = ['Plugin',
            'LOGGER']
 
 
-LOGGER = logging.getLogger('pulsar.test')
 STDOUT_LINE = '\nStdout:\n%s'
 STDERR_LINE = '\nStderr:\n%s'
 
@@ -63,12 +64,14 @@ it will stop the configuration of all subsequent plugins and quit the test.
         return self.__class__.__name__.lower()
 
     def on_start(self):
-        '''Called once by :class:`TestSuite` after all tests are loaded but
-but before it starts running them.'''
+        '''Called by :ref:`TestSuite <test-suite>` once only at startup.
+        
+This callback is invoked once all tests are loaded but but before the test
+suite starts running them.'''
         pass
 
     def on_end(self):
-        '''Called once by :class:`TestSuite` just before it stops.'''
+        '''Called by :ref:`TestSuite <test-suite>` just before it stops.'''
         pass
 
     def loadTestsFromTestCase(self, test_class):
@@ -87,14 +90,13 @@ after the ``_post_teardown`` method.'''
         pass
     
     def before_test_function_run(self, test, local):
-        '''This function can be used by plugins to manipulate the *test*
+        '''This function can be used by plugins to manipulate the ``test``
 behaviour in the process domain where the test run.'''
         return test
     
     def after_test_function_run(self, test, local, result):
-        '''Given a test-function instance return a, possibly, modified test
-instance. This function can be used by plugins to modify the behaviour of test
-cases. By default it returns *test*.'''
+        '''Executed in the ``test`` process domain, after the ``test`` has
+finished.'''
         pass
 
     def addSuccess(self, test):
@@ -105,6 +107,9 @@ cases. By default it returns *test*.'''
 
     def addError(self, test, err):
         pass
+    
+    def addExpectedFailure(self, test, err):
+        pass
 
     def addSkip(self, test, reason):
         pass
@@ -368,7 +373,7 @@ def testsafe(name, return_val=None):
         for p in self.plugins:
             try:
                 c = getattr(p, name)(*args)
-                if c:
+                if c is not None:
                     return return_val(c)
             except Exception:
                 LOGGER.critical('Unhadled error in %s.%s' % (p, name),
@@ -404,6 +409,7 @@ class TestRunner(TestResultProxy):
     stopTest = testsafe('stopTest')
     addSuccess = testsafe('addSuccess')
     addFailure = testsafe('addFailure')
+    addExpectedFailure = testsafe('addExpectedFailure')
     addError = testsafe('addError')
     addSkip = testsafe('addSkip')
     printErrors = testsafe('printErrors')
@@ -445,7 +451,7 @@ class TestRunner(TestResultProxy):
         return result
             
     def run_test_function(self, test, func, timeout=None):
-        '''Run function *func* which belong to *test*.
+        '''Run function ``func`` which belong to ``test``.
     
 :parameter test: test instance or class
 :parameter func: test function belonging to *test*
