@@ -124,7 +124,7 @@ from collections import Mapping
 from functools import partial
 from copy import copy
 
-from pulsar import Deferred, multi_async, is_async, maybe_async, is_failure, async
+from pulsar import Deferred, multi_async, maybe_async, is_failure, async
 from pulsar.utils.pep import iteritems, is_string, ispy3k
 from pulsar.utils.structures import AttributeDictionary, OrderedDict
 from pulsar.utils.html import slugify, INLINE_TAGS, tag_attributes, attr_iter,\
@@ -293,7 +293,7 @@ This is a shortcut for the :meth:`insert` method at index 0.
 This method can be called once only since it invokes the :meth:`stream`
 method.'''
         res = self.stream(request)
-        if is_async(res):
+        if isinstance(res, Deferred):
             return res.add_callback(lambda r: StreamRenderer(r, self.to_string))
         else:
             return StreamRenderer(res, self.to_string)
@@ -361,15 +361,25 @@ This is useful during testing. It is the synchronous equivalent of
 :meth:`content`.'''
         value = maybe_async(self.content(request))
         if is_failure(value):
-            value.raise_all()
+            value.throw()
         return value
             
 
 class Json(AsyncString):
     '''An :class:`AsyncString` which renders into a json string.
 The :attr:`AsyncString.content_type` attribute is set to
-``application/json``.'''
+``application/json``.
+
+.. attribute:: as_list
+    
+    If ``True``, the content is always a list of objects. Default ``False``.
+    
+.. attribute:: parameters
+
+    Additional dictionary of parameters passed during initialisation.
+'''
     def __init__(self, *children, **params):
+        self.as_list = params.pop('as_list', False)
         self.parameters = AttributeDictionary(params)
         for child in children:
             self.append(child)
@@ -386,7 +396,7 @@ is used.'''
         return 'application/json'
         
     def to_string(self, stream):
-        if len(stream) == 1:
+        if len(stream) == 1 and not self.as_list:
             return self.json.dumps(stream[0])
         else:
             return self.json.dumps(stream)
