@@ -1,4 +1,9 @@
-'''Pulsar is shipped with four WSGI application handlers which facilitate the
+'''
+===============================
+WSGI application handlers
+===============================
+
+Pulsar is shipped with two WSGI application handlers to facilitate the
 development of server-side python web applications.
 
 .. note::
@@ -12,7 +17,7 @@ development of server-side python web applications.
 WsgiHandler
 ======================
 
-The first and most basic application handler is the :class:`WsgiHandler`
+The first application handler is the :class:`WsgiHandler`
 which is a step above the :ref:`hello callable <tutorials-hello-world>`
 in the tutorial. It accepts two iterables, a list of
 :ref:`wsgi middleware <wsgi-middleware>` and an optional list of
@@ -34,14 +39,17 @@ Lazy Wsgi Handler
    
 .. _wsgi-middleware:
 
+=====================
 WSGI Middleware
 =====================
 
-Middleware are function or callable objects similar to :ref:`wsgi-handlers`
+Middleware are functions or callable objects similar to
+:ref:`WSGI application handlers <wsgi-handlers>`
 with the only difference that they can return ``None``. Middleware can be used
-in conjunction with :class:`WsgiHandler` or any other handler which iterate
-through middleware in a similar way (for example django wsgi handler).
-Here we introduce the :class:`Router` and :class:`MediaRouter` which handle
+in conjunction with a :class:`WsgiHandler` or any other handler which iterate
+through a list of middleware in a similar way (for example django wsgi handler).
+
+Here we introduce the :class:`Router` and :class:`MediaRouter` to handle
 requests on given urls. Pulsar is shipped with
 :ref:`additional wsgi middleware <wsgi-additional-middleware>` for manipulating
 the environment before a client response is returned.
@@ -49,18 +57,18 @@ the environment before a client response is returned.
 .. _wsgi-router:
 
 Router
-~~~~~~~~~~~~~~~~~~
+=====================
 
-Next up is routing. Routing is the process of match and
-parse the URL to something we can use. Pulsar provides a flexible integrated
+Routing is the process of matching and parsing a URL to something we can use.
+Pulsar provides a flexible integrated
 routing system you can use for that. It works by creating a
 :class:`Router` instance with its own ``rule`` and, optionally, additional
 sub-routers for handling additional urls::
 
     class Page(Router):
-        accept_content_types = ('text/html',
-                                'text/plain',
-                                'application/json')
+        response_content_types = RouterParam(('text/html',
+                                              'text/plain',
+                                              'application/json'))
         
         def get(self, request):
             "This method handle request with get-method" 
@@ -73,7 +81,7 @@ sub-routers for handling additional urls::
     middleware = Page('/bla')
     
 The ``middleware`` constructed can be used to serve ``get`` and ``post`` methods
-at ``/bla``.
+at ``/bla`` url.
 The :class:`Router` introduces a new element into pulsar WSGI handlers, the
 :ref:`wsgi request <app-wsgi-request>`, a light-weight wrapper of the
 WSGI environ.
@@ -84,9 +92,9 @@ WSGI environ.
 
 
 Media Router
-~~~~~~~~~~~~~~~~~~~~~~~~
+=====================
 
-The :class:`MediaRouter` is a spcialised :class:`Router` for serving static
+The :class:`MediaRouter` is a specialised :class:`Router` for serving static
 files such ass ``css``, ``javascript``, images and so forth.
 
 .. autoclass:: MediaRouter
@@ -94,6 +102,13 @@ files such ass ``css``, ``javascript``, images and so forth.
    :member-order: bysource
    
    
+RouterParam
+=================
+
+.. autoclass:: RouterParam
+   :members:
+   :member-order: bysource
+   
 .. _WSGI: http://www.wsgi.org
 '''
 import sys
@@ -107,8 +122,8 @@ from email.utils import parsedate_tz, mktime_tz
 from pulsar.utils.httpurl import http_date, CacheControl
 from pulsar.utils.structures import AttributeDictionary, OrderedDict
 from pulsar.utils.log import LocalMixin, local_property
-from pulsar import Http404, PermissionDenied, HttpException, HttpRedirect,\
-                    async, is_async, multi_async, maybe_async
+from pulsar import (Http404, PermissionDenied, HttpException, HttpRedirect,
+                    async, Failure, multi_async, coroutine_return)
 
 from .route import Route
 from .utils import wsgi_request, handle_wsgi_error
@@ -147,28 +162,24 @@ class WsgiHandler(object):
         self.middleware = middleware or []
         self.response_middleware = response_middleware or []
 
+    @async(get_result=True)
     def __call__(self, environ, start_response):
         '''The WSGI callable'''
-        return maybe_async(self._call(environ, start_response))
-    
-    def _call(self, environ, start_response):
         resp = None
         for middleware in self.middleware:
             try:
-                resp = middleware(environ, start_response)
+                resp = yield middleware(environ, start_response)
             except Exception:
-                resp = handle_wsgi_error(environ, maybe_async(sys.exc_info()))
+                resp = yield handle_wsgi_error(environ, Failure(sys.exc_info()))
             if resp is not None:
                 break
         if resp is None:
             raise Http404
-        if is_async(resp):
-            resp = yield resp.add_errback(partial(handle_wsgi_error, environ))
         if isinstance(resp, WsgiResponse):
             for middleware in self.response_middleware:
                 resp = yield middleware(environ, resp)
             start_response(resp.status, resp.get_headers())
-        yield resp
+        coroutine_return(resp)
     
 
 class LazyWsgi(LocalMixin):
@@ -176,7 +187,7 @@ class LazyWsgi(LocalMixin):
 handler the first time it is called. Subclasses must implement
 the :meth:`setup` method.
 Useful when working in multiprocessing mode when the application
-handler must be a ``pickable`` instance. This handler can rebuild
+handler must be a ``picklable`` instance. This handler can rebuild
 its wsgi :attr:`handler` every time is pickled and un-pickled without
 causing serialisation issues.'''        
     def __call__(self, environ, start_response):
@@ -204,24 +215,37 @@ def get_roule_methods(attrs):
         rule_method = getattr(callable, 'rule_method', None)
         if isinstance(rule_method, tuple):
             rule_methods.append((code, rule_method))
-    return sorted(rule_methods, key=lambda x: x[1][3])
+    return sorted(rule_methods, key=lambda x: x[1].order)
     
 
 class RouterParam(object):
-    
+    '''A :class:`RouterParam` is a way to flag a :class:`Router` parameter
+so that children can retrieve the value if they don't define their own.
+
+A :class:`RouterParam` is always defined as a class attribute and it
+is processed by the :class:`Router` metaclass and stored in a dictionary
+available as ``parameter`` class attribute.
+
+.. attribute:: value
+
+    The value associated with this :class:`RouterParam`. THis is the value
+    stored in the :class:`Router.parameters` dictionary at key given by
+    the class attribute specified in the class definition.
+'''
     def __init__(self, value):
         self.value = value
         
         
 class RouterType(type):
-    
+    ''':class:`Router` metaclass.'''
     def __new__(cls, name, bases, attrs):
         rule_methods = get_roule_methods(attrs.items())
         parameters = {}
-        for name, value in list(attrs.items()):
+        for key, value in list(attrs.items()):
             if isinstance(value, RouterParam):
-                parameters[name] = attrs.pop(name).value
+                parameters[key] = attrs.pop(key).value
         no_rule = set(attrs) - set((x[0] for x in rule_methods))
+        base_rules = []
         for base in reversed(bases):
             if hasattr(base, 'parameters'):
                 params = base.parameters.copy()
@@ -230,11 +254,20 @@ class RouterType(type):
             if hasattr(base, 'rule_methods'):
                 items = base.rule_methods.items()
             else:
-                g = ((name, getattr(base, name)) for name in dir(base))
+                g = ((key, getattr(base, key)) for key in dir(base))
                 items = get_roule_methods(g)
-            base_rules = [pair for pair in items if pair[0] not in no_rule]
-            rule_methods = base_rules + rule_methods
-        #                
+            rules = [pair for pair in items if pair[0] not in no_rule]
+            base_rules = base_rules + rules
+            
+        if base_rules:
+            all = base_rules + rule_methods
+            rule_methods = {}
+            for name, rule in all:
+                if name in rule_methods:
+                    rule = rule.override(rule_methods[name])
+                rule_methods[name] = rule
+            rule_methods = sorted(rule_methods.items(),
+                                  key=lambda x: x[1].order)
         attrs['rule_methods'] = OrderedDict(rule_methods)
         attrs['parameters'] = parameters
         return super(RouterType, cls).__new__(cls, name, bases, attrs)
@@ -260,7 +293,8 @@ request, the ``get(self, request)`` method must be implemented.
 :param routes: Optionals :class:`Router` instances which are added to the
     children :attr:`routes` of this router.
 :param parameters: Optional parameters for this router. They are stored in the
-    :attr:`parameters` attribute.
+    :attr:`parameters` attribute. If a ``response_content_types`` value is
+    passed, it overrides the :attr:`response_content_types` attribute.
     
 .. attribute:: route
 
@@ -274,6 +308,13 @@ request, the ``get(self, request)`` method must be implemented.
 
     The parent :class:`Router` of this :class:`Router`.
     
+.. attribute:: response_content_types
+
+    a list/tuple of possible content types of a response to a client request.
+    
+    The client request must accept at least one of the response content
+    types, otherwise an HTTP ``415`` exception occurs.
+    
 .. attribute:: parameters
 
     A :class:`pulsar.utils.structures.AttributeDictionary` of parameters for
@@ -285,7 +326,7 @@ request, the ``get(self, request)`` method must be implemented.
     _parent = None
     _name = None
     
-    accept_content_types = RouterParam(None)
+    response_content_types = RouterParam(None)
     
     def __init__(self, rule, *routes, **parameters):
         Router._creation_count += 1
@@ -299,7 +340,7 @@ request, the ``get(self, request)`` method must be implemented.
             self.add_child(router)
         self.parameters = AttributeDictionary(self.parameters)
         for name, rule_method in self.rule_methods.items():
-            rule, method, params, count = rule_method
+            rule, method, params, _, _ = rule_method
             rparameters = params.copy()
             handler = getattr(self, name)
             if rparameters.pop('async', False): # asynchronous method
@@ -316,7 +357,8 @@ request, the ``get(self, request)`` method must be implemented.
     @property
     def name(self):
         '''The name of this :class:`Router`. This attribute can be specified
-during initialisation.'''
+during initialisation. If available, it can be used to retrieve a child router
+by name via the :meth:`get_route` method.'''
         return self._name
     
     @property
@@ -331,12 +373,37 @@ during initialisation.'''
     def parent(self):
         return self._parent
     
+    @property
+    def default_content_type(self):
+        '''The default content type for responses. This is the first element
+in the :attr:`response_content_types` list.'''
+        ct = self.response_content_types
+        return ct[0] if ct else None
+    
     @property
     def creation_count(self):
-        '''Integer for sorting :class:`Router` by creation.'''
+        '''Integer for sorting :class:`Router` by creation. It is autogenerated
+during :class:`Router` initialisation.'''
         return self._creation_count
+    
+    @property
+    def full_route(self):
+        '''The full :attr:`route` for this :class:`Router`. It includes the
+:attr:`parent` portion of the route if a parent router is available.'''
+        route = self.route
+        if self._parent:
+            route = self._parent.route + route
+        return route
+        
+    @property
+    def rule(self):
+        '''The full ``rule`` string for this :class:`Router`. It includes the
+:attr:`parent` portion of rule if a parent router is available.'''
+        return self.full_route.rule
         
     def path(self, **urlargs):
+        '''The full path of this :class:`Router`. It includes the
+:attr:`parent` portion of url if a parent router is available.'''
         route = self.route
         if self._parent:
             route = self._parent.route + route
@@ -367,13 +434,14 @@ during initialisation.'''
                              (self.__class__.__name__, name))
         
     def content_type(self, request):
-        '''Evaluate the content type for the response to the client ``request``.
-The method uses the :attr:`accept_content_types` tuple of accepted content
+        '''Evaluate the content type for the response to a client ``request``.
+
+The method uses the :attr:`response_content_types` parameter of accepted content
 types and the content types accepted by the client and figure out
 the best match.'''
-        accept = self.accept_content_types
-        if accept:
-            return request.content_types.best_match(accept)
+        response_content_types = self.response_content_types
+        if response_content_types:
+            return request.content_types.best_match(response_content_types)
        
     def __repr__(self):
         return self.route.__repr__()
@@ -418,7 +486,7 @@ the best match.'''
         else:
             return self, match
     
-    @async()
+    @async(get_result=True)
     def response(self, environ, start_response, args):
         '''Once the :meth:`resolve` method has matched the correct
 :class:`Router` for serving the request, this matched router invokes this method
@@ -432,10 +500,19 @@ to actually produce the WSGI response.'''
             raise HttpException(status=405,
                                 msg='Method "%s" not allowed' % method)
         # make sure cache does not contain asynchronous data
-        environ['pulsar.cache'] = yield multi_async(request.cache)
-        yield callable(request)
+        async_cache = multi_async(request.cache, raise_on_error=False)
+        cache = yield async_cache
+        if async_cache.num_failures:
+            for key, value in list(cache.items()):
+                if isinstance(value, Failure):
+                    cache.pop(key)
+            environ['pulsar.cache'] = cache
+            yield async_cache.failures
+        else:
+            environ['pulsar.cache'] = cache
+            yield callable(request)
     
-    @async()
+    @async(get_result=True)
     def redirect(self, environ, start_response, path):
         request = wsgi_request(environ, self)
         environ['pulsar.cache'] = yield multi_async(request.cache)
@@ -510,7 +587,8 @@ returns ``utf-8``.'''
     
 
 class MediaMixin(Router):
-    accept_content_types = ('application/octet-stream', 'text/css')
+    response_content_types = RouterParam(('application/octet-stream',
+                                          'text/css'))
     cache_control = CacheControl(maxage=86400)
     _file_path = ''
         
