@@ -33,12 +33,13 @@ class HtmlVisitor(HtmlType('HtmlVisitorBase', (object,), {'abstract': True})):
         html.attr('value', value)
 
     def add_data(self, html, key, value):
-        data = html._data
-        if key in data and isinstance(value, Mapping):
-            target = data[key]
-            if isinstance(target, Mapping):
-                return recursive_update(target, value)
-        data[key] = value
+        if value is not None:
+            data = html._data
+            if key in data and isinstance(value, Mapping):
+                target = data[key]
+                if isinstance(target, Mapping):
+                    return recursive_update(target, value)
+            data[key] = value
         
 
 base = HtmlVisitor()
@@ -70,12 +71,12 @@ class Select(HtmlVisitor):
     def set_form_value(self, html, value):
         if html.attr('multiple') == 'multiple':
             for child in html.children:
-                if child.attr('value' == value):
+                if child.attr('value') == value:
                     child.attr('selected', 'selected')
                     break
         else:
             for child in html.children:
-                if child.attr('value' == value):
+                if child.attr('value') == value:
                     child.attr('selected', 'selected')
                 else:
                     child._attr.pop('selected', None)
