@@ -1,9 +1,16 @@
 '''
+HTTP Protocol Consumer
+==============================
 
 .. autoclass:: HttpServerResponse
    :members:
    :member-order: bysource
    
+   
+Testing WSGI Environ
+=========================
+
+.. autofunction:: test_wsgi_environ
 '''
 import sys
 import time
@@ -14,16 +21,16 @@ from wsgiref.handlers import format_date_time
 from io import BytesIO
 
 import pulsar
-from pulsar import HttpException, ProtocolError, maybe_async, is_async,\
-                   is_failure
-from pulsar.utils.pep import is_string, to_bytes, native_str
-from pulsar.utils.httpurl import Headers, unquote, has_empty_content,\
-                                 host_and_port_default, Headers, http_parser,\
-                                 REDIRECT_CODES
+from pulsar import HttpException, ProtocolError, maybe_async, Deferred, Failure
+from pulsar.utils.pep import is_string, native_str, raise_error_trace
+from pulsar.utils.httpurl import (Headers, unquote, has_empty_content,
+                                  host_and_port_default, http_parser)
+                                 
+from pulsar.utils.internet import format_address
+from pulsar.async.protocols import ProtocolConsumer
 from pulsar.utils import events
 
 from .utils import handle_wsgi_error, LOGGER, HOP_HEADERS
-from .wrappers import WsgiResponse
 
 
 __all__ = ['HttpServerResponse', 'MAX_CHUNK_SIZE', 'test_wsgi_environ']
@@ -31,8 +38,18 @@ __all__ = ['HttpServerResponse', 'MAX_CHUNK_SIZE', 'test_wsgi_environ']
 MAX_CHUNK_SIZE = 65536
 
 
-def test_wsgi_environ(url='/', method='GET', headers=None, extra=None):
+def test_wsgi_environ(url='/', method=None, headers=None, extra=None):
+    '''An utility function for creating a WSGI environment dictionary
+for testing purposes.
+    
+:param url: the resource in the ``PATH_INFO``.
+:param method: the ``REQUEST_METHOD``.
+:param headers: optional request headers
+:param extra: optional dictionary of additional key-valued parameters to add.
+:return: a valid WSGI environ dictionary.
+'''
     parser = http_parser(kind=0)
+    method = (method or 'GET').upper()
     data = '%s %s HTTP/1.1\r\n\r\n' % (method, url)
     data = data.encode('utf-8')
     parser.execute(data, len(data))
@@ -61,7 +78,7 @@ def wsgi_environ(parser, address, client_address, request_headers,
         }
     url_scheme = "http"
     forward = client_address
-    server = '%s:%s' % address
+    server = format_address(address)
     script_name = os.environ.get("SCRIPT_NAME", "")
     for header, value in request_headers:
         header = header.lower()
@@ -146,7 +163,7 @@ def keep_alive_with_status(status, headers):
     return True
     
 
-class HttpServerResponse(pulsar.ProtocolConsumer):
+class HttpServerResponse(ProtocolConsumer):
     '''Server side HTTP :class:`pulsar.ProtocolConsumer`.'''
     _status = None
     _headers_sent = None
@@ -218,11 +235,12 @@ the following algorithm:
         '''WSGI compliant ``start_response`` callable, see pep3333_.
 The application may call start_response more than once, if and only
 if the ``exc_info`` argument is provided.
-More precisely, it is a fatal error to call start_response without the exc_info
-argument if start_response has already been called within the current
-invocation of the application.
+More precisely, it is a fatal error to call ``start_response`` without the
+``exc_info`` argument if start_response has already been called within
+the current invocation of the application.
 
-:parameter status: an HTTP "status" string like "200 OK" or "404 Not Found".
+:parameter status: an HTTP ``status`` string like ``200 OK`` or
+    ``404 Not Found``.
 :parameter response_headers: a list of ``(header_name, header_value)`` tuples.
     It must be a Python list. Each header_name must be a valid HTTP header
     field-name (as defined by RFC 2616_, Section 4.2), without a trailing
@@ -230,10 +248,9 @@ invocation of the application.
 :parameter exc_info: optional python ``sys.exc_info()`` tuple. This argument
     should be supplied by the application only if start_response is being
     called by an error handler.
+:return: The :meth:`write` method.
 
-:rtype: The :meth:`HttpResponse.write` callable.
-
-The ``HOP_HEADERS`` are not considered but no error is raised.
+``HOP_HEADERS`` are not considered but no error is raised.
 
 .. _pep3333: http://www.python.org/dev/peps/pep-3333/
 .. _2616: http://www.faqs.org/rfcs/rfc2616.html
@@ -244,7 +261,7 @@ The ``HOP_HEADERS`` are not considered but no error is raised.
                     # if exc_info is provided, and the HTTP headers have
                     # already been sent, start_response must raise an error,
                     # and should re-raise using the exc_info tuple
-                    raise (exc_info[0], exc_info[1], exc_info[2])
+                    raise_error_trace(exc_info[1], exc_info[2])
             finally:
                 # Avoid circular reference
                 exc_info = None
@@ -301,7 +318,7 @@ method as required by the WSGI specification.
         except Exception:
             result = sys.exc_info()
         else:
-            if not is_failure(wsgi_iter):
+            if not isinstance(wsgi_iter, Failure):
                 result = self.async_wsgi(wsgi_iter)
             else:
                 result = wsgi_iter
@@ -312,7 +329,7 @@ method as required by the WSGI specification.
     def async_wsgi(self, wsgi_iter):
         '''Asynchronous WSGI server handler. Fully conforms with `WSGI 1.0.1`_
 when the ``wsgi_iter`` yields bytes only.'''
-        if is_async(wsgi_iter): # handle asynchronous wsgi response
+        if isinstance(wsgi_iter, Deferred): # handle asynchronous wsgi response
             wsgi_iter = yield wsgi_iter 
         try:
             for b in wsgi_iter:
