@@ -2,7 +2,6 @@ import threading
 import logging
 from threading import current_thread
 from multiprocessing import current_process
-from multiprocessing.dummy import DummyProcess
 
 from pulsar.utils.pep import get_event_loop_policy
 
@@ -11,20 +10,24 @@ __all__ = ['get_request_loop',
            'is_mainthread',
            'process_local_data',
            'thread_local_data',
+           'logger',
            'NOTHING']
 
 LOGGER = logging.getLogger('pulsar')
 NOTHING = object()
 
 def is_mainthread(thread=None):
-    '''Check if thread is the main thread. If *thread* is not supplied check
-the current thread'''
+    '''Check if thread is the main thread. If ``thread`` is not supplied check
+the current thread.'''
     thread = thread if thread is not None else current_thread() 
     return isinstance(thread, threading._MainThread)
 
 def get_request_loop():
     return get_event_loop_policy().get_request_loop()
 
+def logger(event_loop=None):
+    return getattr(event_loop or get_request_loop(), 'logger', LOGGER)
+
 def process_local_data(name=None):
     '''Fetch the current process local data dictionary. If *name* is not
 ``None`` it returns the value at *name*, otherwise it return the process data
@@ -63,30 +66,23 @@ get_actor = lambda: thread_local_data('actor')
 def set_actor(actor):
     '''Set and returns the actor running the current thread.'''
     actor = thread_local_data('actor', value=actor)
-    if actor.impl.kind == 'thread':
-        process_local_data('thread_actors')[actor.aid] = actor
+    #if actor.impl.kind == 'thread':
+    #    process_local_data('thread_actors')[actor.aid] = actor
     return actor
 
 def remove_actor(actor):
     '''Remove actor from threaded_actors dictionary'''
     if actor.impl.kind == 'thread':
         LOGGER.debug('Removing threaded actor %s' % actor)
-        process_local_data('thread_actors').pop(actor.aid, None)
-
-def get_actor_from_id(aid):
-    '''Retrieve an actor from its actor id. This function can be used by
-actors with thread concurrency ince they live in the arbiter process domain.'''
-    actors = process_local_data('thread_actors')
-    if actors:
-        return actors.get(aid)
+        #process_local_data('thread_actors').pop(actor.aid, None)
         
         
 class plocal(object):
     pass
         
 class ProcessLocal(object):
-    def __init__(self):
-        self.thread_actors = {}
+    #def __init__(self):
+    #    self.thread_actors = {}
         
     def local(self):
         return plocal()
\ No newline at end of file
