@@ -1,9 +1,5 @@
 import os
 import sys
-import logging
-import traceback
-import inspect
-import errno
 import socket
 from heapq import heappush, heappop
 from functools import partial
@@ -14,21 +10,23 @@ try:
 except ImportError: #pragma    nocover
     signal = None
 
-from pulsar.utils.system import IObase, IOpoll, close_on_exec, platform, Waker
-from pulsar.utils.pep import default_timer, set_event_loop_policy,\
-                             set_event_loop, new_event_loop, get_event_loop,\
-                             EventLoop as BaseEventLoop, range,\
-                             EventLoopPolicy as BaseEventLoopPolicy
-from pulsar.utils.sockets import SOCKET_INTERRUPT_ERRORS
+from pulsar.utils.system import close_on_exec
+from pulsar.utils.pep import (default_timer, set_event_loop_policy,
+                              set_event_loop, range,
+                              EventLoop as BaseEventLoop, 
+                              EventLoopPolicy as BaseEventLoopPolicy)
+from pulsar.utils.internet import SOCKET_INTERRUPT_ERRORS
+from pulsar.utils.exceptions import StopEventLoop, ImproperlyConfigured
 
-from .access import thread_local_data
-from .defer import log_failure, is_failure, Deferred, TimeoutError, maybe_async
-from .transports import create_server
+from .access import thread_local_data, LOGGER
+from .defer import Failure, TimeoutError, maybe_async
+from .stream import create_connection, start_serving, sock_connect, sock_accept
+from .udp import create_datagram_endpoint
+from .consts import DEFAULT_CONNECT_TIMEOUT, DEFAULT_ACCEPT_TIMEOUT
+from .pollers import DefaultIO
 
 __all__ = ['EventLoop', 'TimedCall']
 
-LOGGER = logging.getLogger('pulsar.eventloop')
-
 
 def file_descriptor(fd):
     if hasattr(fd, 'fileno'):
@@ -42,10 +40,7 @@ def setid(self):
     self.pid = os.getpid()
     return ct
 
-
-class StopEventLoop(BaseException):
-    """Raised to stop the event loop."""
-    
+STOP_LOOP = (KeyboardInterrupt, )
     
 class EventLoopPolicy(BaseEventLoopPolicy):
     '''Pulsar event loop policy'''
@@ -124,129 +119,6 @@ it is created by :meth:`EventLoop.call_soon`, :meth:`EventLoop.call_later`,
             args = self._args + args
             return self._callback(*args, **kwargs)
         
-        
-class FileDescriptor(IObase):
-    def __init__(self, fd, eventloop):
-        self.fd = fd
-        self.eventloop = eventloop
-        self.logger = eventloop.logger
-        self.handle_write = None
-        self.handle_read = None
-
-    @property
-    def poller(self):
-        return self.eventloop._impl
-    
-    @property
-    def reading(self):
-        return bool(self.handle_read)
-    
-    @property
-    def writing(self):
-        return bool(self.handle_write)
-    
-    @property
-    def state(self):
-        if self.reading:
-            if self.writing:
-                return self.READ | self.WRITE
-            else:
-                return self.READ
-        elif self.writing:
-            return self.WRITE
-            
-    @property
-    def state_code(self):
-        s = []
-        if self.state is None:
-            return 'closed'
-        if self.connecting:
-            s.append('connecting')
-        elif self.writing:
-            s.append('writing')
-        if self.reading:
-            s.append('reading')
-        return ' '.join(s) if s else 'idle'
-    
-    def add_connector(self, callback):
-        self.add_writer(callback, 'connector')
-        
-    def add_reader(self, callback):
-        handle_read = self.handle_read
-        if handle_read:
-            if handle_read.callback != callback.callback:
-                raise RuntimeError('Read handler already registered')
-        else:
-            self.logger.debug('Add reader on file descriptor %s', self.fd)
-            current_state = self.state
-            self.handle_read = callback
-            self.modify_state(current_state, self.READ)
-        
-    def add_writer(self, callback, name=None):
-        name = name or 'writer'
-        handle_write = self.handle_write
-        if handle_write:
-            if handle_write.callback != callback.callback:
-                raise RuntimeError('%s handler already registered', name)
-        else:
-            self.logger.debug('Add %s on file descriptor %s', name, self.fd)
-            current_state = self.state
-            self.handle_write = callback
-            self.modify_state(current_state, self.WRITE)
-        
-    def remove_connector(self):
-        return self.remove_writer('connector')
-    
-    def remove_reader(self):
-        '''Remove reader and return True if writing'''
-        if self.writing:
-            self.logger.debug('Remove reader from file descriptor %s', self.fd)
-            self.poller.modify(self.fd, self.WRITE)
-        else:
-            self.remove_handler()
-        self.handle_read = None
-
-    def remove_writer(self, name=None):
-        '''Remove writer and return True if reading'''
-        if self.reading:
-            self.logger.debug('Remove %s from file descriptor %s',
-                              name or 'writer', self.fd)
-            self.poller.modify(self.fd, self.READ)
-        else:
-            self.remove_handler()
-        self.handle_write = None
-    
-    def __call__(self, events):
-        if events & self.READ:
-            if self.handle_read:
-                log_failure(self.handle_read())
-            else:
-                self.logger.warning('Read callback without handler for file'
-                                    ' descriptor %s.', self.fd)
-        if events & self.WRITE:
-            if self.handle_write:
-                log_failure(self.handle_write())
-            else:
-                self.logger.warning('Write callback without handler for file'
-                                    ' descriptor %s.', self.fd)
-            
-    def modify_state(self, current_state, state):
-        if current_state != state:
-            if current_state is None:
-                self.poller.register(self.fd, state)
-            else:
-                self.poller.modify(self.fd, current_state | state)
-    
-    def remove_handler(self):
-        """Stop listening for events on fd."""
-        fd = self.fd
-        try:
-            self.poller.unregister(fd)
-        except (OSError, IOError):
-            self.eventloop.logger.error("Error removing %s from EventLoop", fd)
-        self.logger.debug('Remove file descriptor %s', fd)
-        self.eventloop._handlers.pop(fd, None)
-
 
 class LoopingCall(object):
     
@@ -270,14 +142,15 @@ class LoopingCall(object):
     def cancel(self, result=None):
         '''Attempt to cancel the callback.'''
         self._cancelled = True
-        log_failure(result)
         
     def __call__(self):
-        # If this fails, the chain is broken.
         try:
-            result = maybe_async(self.callback(*self.args), get_result=False)
+            result = maybe_async(self.callback(*self.args),
+                                 get_result=False,
+                                 event_loop=self.event_loop)
         except Exception:
-            self.cancel(sys.exc_info())
+            result = Failure(sys.exc_info())
+            self.cancel(result)
         else:
             result.add_callback(self._continue, self.cancel)
         
@@ -292,36 +165,21 @@ class LoopingCall(object):
                 self.event_loop._callbacks.append(self.handler)
                 
 
-class EventLoop(IObase, BaseEventLoop):
+class EventLoop(BaseEventLoop):
     """A pluggable event loop which conforms with the pep-3156_ API. The
 event loop is the place where most asynchronous operations are carried out.
 
-.. attribute:: io
-
-    The I/O implementation. If not supplied, the best possible
-    implementation available will be used. On posix system this is ``epoll``,
-    or else ``select``. It can be any other custom implementation as long as
-    it has an ``epoll`` like interface.
-
-.. attribute:: cpubound
-
-    If ``True`` this is a CPU bound event loop, otherwise it is an I/O
-    event loop. CPU bound loops can block the loop for considerable amount
-    of time.
-        
-.. attribute:: num_loops
-
-    Total number of loops
-
 .. attribute:: poll_timeout
 
-    The timeout in seconds when polling with epol or select.
+    The timeout in seconds when polling with ``epolL``, ``kqueue``, ``select``
+    and so forth.
 
-    Default: `0.5`
+    Default: ``0.5``
 
 .. attribute:: tid
 
-    The thread id where the eventloop is running
+    The thread id where this event loop is running. If the event loop is not
+    running this attribute is ``None``.
 
 """
     # Never use an infinite timeout here - it can stall epoll
@@ -331,20 +189,19 @@ event loop is the place where most asynchronous operations are carried out.
 
     def __init__(self, io=None, logger=None, poll_timeout=None, timer=None,
                  iothreadloop=True):
-        self._impl = io or IOpoll()
-        self.fd_factory = getattr(self._impl, 'fd_factory', FileDescriptor)
+        self._io = io or DefaultIO()
         self.timer = timer or default_timer
         self.poll_timeout = poll_timeout if poll_timeout else self.poll_timeout
         self.logger = logger or LOGGER
-        close_on_exec(self._impl.fileno())
+        close_on_exec(self._io.fileno())
         self._iothreadloop = iothreadloop
         self._handlers = {}
         self._callbacks = deque()
         self._scheduled = []
         self._name = None
-        self.num_loops = 0
+        self._num_loops = 0
         self._default_executor = None
-        self._waker = self.install_waker()
+        self._waker = self._io.install_waker(self)
     
     def __repr__(self):
         return self.name
@@ -360,7 +217,11 @@ event loop is the place where most asynchronous operations are carried out.
             
     @property
     def io(self):
-        return self._impl
+        '''The :class:`Poller` for this event loop. If not supplied,
+        the best possible implementation available will be used. On posix
+        system this is ``epoll`` or ``kqueue`` (Mac OS)
+        or else ``select``.'''
+        return self._io
     
     @property
     def iothreadloop(self):
@@ -370,7 +231,10 @@ loop of the thread where it is run.'''
     
     @property
     def cpubound(self):
-        return getattr(self._impl, 'cpubound', False)
+        '''If ``True`` this is a CPU bound event loop, otherwise it is an I/O
+        event loop. CPU bound loops can block the loop for considerable amount
+        of time.'''
+        return getattr(self._io, 'cpubound', False)
     
     @property
     def running(self):
@@ -380,8 +244,12 @@ loop of the thread where it is run.'''
     def active(self):
         return bool(self._callbacks or self._scheduled or self._handlers)
     
-    ############################################################################
-    ##    PEP 3156 Methods
+    @property
+    def num_loops(self):
+        '''Total number of loops.'''
+        return self._num_loops
+    
+    #################################################    STARTING & STOPPING
     def run(self):
         '''Run the event loop until nothing left to do or stop() called.'''
         if not self.running:
@@ -407,21 +275,6 @@ loop of the thread where it is run.'''
                         break
             finally:
                 self._after_run()
-
-    def run_once(self, timeout=None):
-        """Run through all callbacks and all I/O polls once.
-
-        Calling stop() will break out of this too.
-        """
-        if not self.running:
-            self._before_run()
-            try:
-                try:
-                    self._run_once(timeout)
-                except StopEventLoop:
-                    pass
-            finally:
-                self._after_run()
                 
     def run_until_complete(self, future, timeout=None):
         '''Run the event loop until a :class:`Deferred` *future* is done.
@@ -439,15 +292,20 @@ raise TimeoutError (but don't cancel the future).'''
             else:
                 raise TimeoutError
         result = future.result
-        if is_failure(result):
-            result.raise_all()
+        if isinstance(result, Failure):
+            result.throw()
         else:
             return result
-        
+    
     def stop(self):
         '''Stop the loop after the current event loop iteration is complete'''
         self.call_soon_threadsafe(self._raise_stop_event_loop)
     
+    def is_running(self):
+        '''``True`` if the loop is running.'''
+        return bool(self._name)
+    
+    #################################################    CALLBACKS
     def call_at(self, when, callback, *args):
         '''Arrange for a ``callback`` to be called at a given time ``when``
 in the future. The time is an absolute time, for relative time check
@@ -492,6 +350,7 @@ the callback when it is called.'''
         self._callbacks.append(timeout)
         return timeout
     
+    #################################################    THREAD INTERACTION
     def call_soon_threadsafe(self, callback, *args):
         '''Calls the given callback on the next I/O loop iteration.
 It is safe to call this method from any thread at any time.
@@ -504,72 +363,63 @@ to transfer control from other threads to the EventLoop's thread.'''
         return timeout
     
     def run_in_executor(self, executor, callback, *args):
-        '''Arrange to call ``callback(*args)`` in an executor.
-NOT YET SUPPORTED.'''
+        '''Arrange to call ``callback(*args)`` in an ``executor``.
+        
+        Return a :class:`Deferred` called once the callback has finished.'''
         executor = executor or self._default_executor
         if executor is None:
-            from pulsar import ThreadPool
-            executor = ThreadPool(_MAX_WORKERS)
-            self._default_executor = executor
-        return wrap_future(executor.submit(callback, *args), loop=self)
+            raise ImproperlyConfigured('No executor available')
+        return executor.apply(callback, *args)
         
     def set_default_executor(self, executor):
         self._default_executor = executor
-        
+    
+    #################################################    INTERNET NAME LOOKUPS
     def getaddrinfo(self, host, port, family=0, type=0, proto=0, flags=0):
-        return self.run_in_executor(None, socket.getaddrinfo,
-                                    host, port, family, type, proto, flags)
+        return socket.getaddrinfo(host, port, family, type, proto, flags)
 
     def getnameinfo(self, sockaddr, flags=0):
-        return self.run_in_executor(None, socket.getnameinfo, sockaddr, flags)
-        
+        return socket.getnameinfo(sockaddr, flags)
+    
+    #################################################    I/O CALLBACKS
     def add_reader(self, fd, callback, *args):
         """Add a reader callback.  Return a Handler instance."""
         handler = TimedCall(None, callback, args)
-        fd = file_descriptor(fd)
-        if fd not in self._handlers:
-            self._handlers[fd] = self.fd_factory(fd, self)
-        self._handlers[fd].add_reader(handler)
+        self._io.add_reader(file_descriptor(fd), handler)
         return handler
     
     def add_writer(self, fd, callback, *args):
         """Add a reader callback.  Return a Handler instance."""
         handler = TimedCall(None, callback, args)
-        fd = file_descriptor(fd)
-        if fd not in self._handlers:
-            self._handlers[fd] = self.fd_factory(fd, self)
-        self._handlers[fd].add_writer(handler)
+        self._io.add_writer(file_descriptor(fd), handler)
         return handler
     
     def add_connector(self, fd, callback, *args):
         handler = TimedCall(None, callback, args)
         fd = file_descriptor(fd)
-        if fd not in self._handlers:
-            self._handlers[fd] = self.fd_factory(fd, self)
-        self._handlers[fd].add_connector(handler)
+        self._io.add_writer(fd, handler)
+        self._io.add_error(fd, handler)
         return handler
     
     def remove_reader(self, fd):
         '''Cancels the current read callback for file descriptor fd,
 if one is set. A no-op if no callback is currently set for the file
 descriptor.'''
-        fd = file_descriptor(fd)
-        if fd in self._handlers:
-            self._handlers[fd].remove_reader()
+        return self._io.remove_reader(file_descriptor(fd))
     
     def remove_writer(self, fd):
         '''Cancels the current write callback for file descriptor fd,
 if one is set. A no-op if no callback is currently set for the file
 descriptor.'''
-        fd = file_descriptor(fd)
-        if fd in self._handlers:
-            self._handlers[fd].remove_writer()
+        return self._io.remove_writer(file_descriptor(fd))
             
     def remove_connector(self, fd):
         fd = file_descriptor(fd)
-        if fd in self._handlers:
-            self._handlers[fd].remove_connector()
+        w = self._io.remove_writer(fd)
+        e = self._io.remove_error(fd)
+        return w or e
     
+    #################################################    SIGNAL CALLBACKS
     def add_signal_handler(self, sig, callback, *args):
         '''Whenever signal ``sig`` is received, arrange for `callback(*args)` to
 be called. Returns a :class:`TimedCall` handler which can be used to cancel
@@ -591,19 +441,108 @@ default signal handler ``signal.SIG_DFL``.'''
             return True
         else:
             return False
-        
-    def create_server(self, **kwargs):
-        '''Create a new :class:`Server`.'''
-        if self.cpubound:
-            raise RuntimeError('Cannot create server from a cpubound eventloop')
-        return create_server(self, **kwargs)
     
+    #################################################    SOCKET METHODS        
+    def create_connection(self, protocol_factory, host=None, port=None,
+                          ssl=None, family=0, proto=0, flags=0, sock=None,
+                          local_addr=None, timeout=None):
+        '''Creates a stream connection to a given internet host and port.
+        
+It is the asynchronous equivalent of ``socket.create_connection``.
+
+:parameter protocol_factory: The callable to create the :class:`Protocol`
+    which handle the connection.
+:parameter host: If host is an empty string or None all interfaces are assumed
+    and a list of multiple sockets will be returned (most likely
+    one for IPv4 and another one for IPv6)
+:parameter port:
+:parameter ssl:
+:parameter family:
+:parameter proto:
+:parameter flags:
+:parameter sock:
+:parameter local_addr: if supplied, it must be a 2-tuple ``(host, port)`` for
+    the socket to bind to as its source address before connecting.
+:return: a :class:`Deferred` and its result on success is the
+    ``(transport, protocol)`` pair.
+    
+If a failure prevents the creation of a successful connection, an appropriate
+exception will be raised.'''
+        timeout = timeout or DEFAULT_CONNECT_TIMEOUT
+        res = create_connection(self, protocol_factory, host, port,
+                                ssl, family, proto, flags, sock, local_addr)
+        return maybe_async(res, event_loop=self, timeout=timeout,
+                           get_result=False)
+    
+    def start_serving(self, protocol_factory, host=None, port=None, ssl=None,
+                      family=socket.AF_UNSPEC, flags=socket.AI_PASSIVE,
+                      sock=None, backlog=100, reuse_address=None):
+        """Creates a TCP server bound to ``host`` and ``port``.
+        
+:parameter protocol_factory: The :class:`Protocol` which handle server requests.
+:parameter host: If host is an empty string or None all interfaces are assumed
+    and a list of multiple sockets will be returned (most likely
+    one for IPv4 and another one for IPv6).
+:parameter port: integer indicating the port number.
+:parameter ssl: can be set to an SSLContext to enable SSL over the accepted
+    connections.
+:parameter family: socket family can be set to either ``AF_INET`` or
+    ``AF_INET6`` to force the socket to use IPv4 or IPv6.
+    If not set it will be determined from host (defaults to ``AF_UNSPEC``).
+:parameter flags: is a bitmask for :meth:`getaddrinfo`.
+:parameter sock: can optionally be specified in order to use a pre-existing
+    socket object.
+:parameter backlog: is the maximum number of queued connections passed to
+    listen() (defaults to 100).
+:parameter reuse_address: tells the kernel to reuse a local socket in
+    ``TIME_WAIT`` state, without waiting for its natural timeout to
+    expire. If not specified will automatically be set to ``True`` on UNIX.
+:return: a :class:`Deferred` whose result will be a list of socket objects
+    which will later be handled by ``protocol_factory``.
+        """
+        res = start_serving(self, protocol_factory, host, port, ssl,
+                            family, flags, sock, backlog, reuse_address)
+        return maybe_async(res, event_loop=self, get_result=False)
+    
+    def create_datagram_endpoint(self, protocol_factory, local_addr=None,
+                                 remote_addr=None, family=socket.AF_UNSPEC,
+                                 proto=0, flags=0):
+        res = create_datagram_endpoint(self, protocol_factory, local_addr,
+                                       remote_addr, family, proto, flags)
+        return maybe_async(res, event_loop=self, get_result=False)
+        
+        
+    def stop_serving(self, sock):
+        '''The argument should be a socket from the list returned by
+:meth:`start_serving` method. The serving loop associated with that socket
+will be stopped.'''
+        self.remove_reader(sock.fileno())
+        sock.close()
+        
+    def sock_connect(self, sock, address, timeout=None):
+        '''Connect ``sock`` to the given ``address``.
+        
+        Returns a :class:`Deferred` whose result on success will be ``None``.'''
+        res = sock_connect(self, sock, address)
+        return maybe_async(res, event_loop=self, timeout=timeout)
+    
+    def sock_accept(self, sock, timeout=None):
+        '''Accept a connection from a socket ``sock``.
+        
+        The socket must be in listening mode and bound to an address.
+        Returns a :class:`Deferred` whose result on success will be a tuple
+        ``(conn, peer)`` where ``conn`` is a connected non-blocking socket
+        and ``peer`` is the peer address.'''
+        timeout = timeout or DEFAULT_ACCEPT_TIMEOUT
+        res = sock_accept(self, sock)
+        return maybe_async(res, event_loop=self, timeout=timeout)
+        
+    #################################################    NON PEP METHODS
     def wake(self):
         '''Wake up the eventloop.'''
         if self.running and self._waker:
             self._waker.wake()
             
-    ############################################################ NON PEP METHODS
     def call_repeatedly(self, interval, callback, *args):
         """Call a ``callback`` every ``interval`` seconds. It handles
 asynchronous results. If an error occur in the ``callback``, the chain is
@@ -611,14 +550,16 @@ broken and the ``callback`` won't be called anymore."""
         return LoopingCall(self, callback, args, interval)
     
     def call_every(self, callback, *args):
-        """Same as :meth:``call_repeatedly`` with the only difference that
-the ``callback`` is scheduled at every loop."""
+        '''Same as :meth:`call_repeatedly` with the only difference that
+the ``callback`` is scheduled at every loop. Installing this callback cause
+the event loop to poll with a 0 timeout all the times.'''
         return LoopingCall(self, callback, args)
-            
+    
     def call_now_threadsafe(self, callback, *args):
         '''Same as :meth:`call_soon_threadsafe` with the only exception that
 if we are calling from the same thread of execution as this
-:class:`EventLoop`, the ``callback`` is called immediately.'''
+:class:`EventLoop`, the ``callback`` is called immediately. Otherwise
+:meth:`call_soon_threadsafe` is invoked.'''
         if self.tid != current_thread().ident:
             return self.call_soon_threadsafe(callback, *args)
         else:
@@ -629,20 +570,8 @@ if we are calling from the same thread of execution as this
             return callback in self._scheduled
         else:
             return callback in self._callbacks
-        
-    def install_waker(self):
-        # Install event loop wake if possible
-        if hasattr(self._impl, 'install_waker'):
-            return self._impl.install_waker(self)
-        else:
-            waker = Waker()
-            self.add_reader(waker, waker.consume)
-            return waker
 
-    ############################################################################
-    ##    INTERNALS
-    ############################################################################
-    
+    #################################################    INTERNALS    
     def _before_run(self):
         ct = setid(self)
         self._name = ct.name
@@ -650,12 +579,11 @@ if we are calling from the same thread of execution as this
             set_event_loop(self)
     
     def _after_run(self):
-        self.logger.debug('Exiting %s', self)
+        self.logger.info('Exiting %s', self)
         self._name = None
         self.tid = None
         
     def _raise_stop_event_loop(self, exc=None):
-        self.logger.debug('Stopping %s', self)
         raise StopEventLoop
 
     def _check_signal(self, sig):
@@ -674,7 +602,7 @@ if we are calling from the same thread of execution as this
 
     def _run_once(self, timeout=None):
         timeout = timeout or self.poll_timeout
-        self.num_loops += 1
+        self._num_loops += 1
         #
         # Compute the desired timeout
         if self._callbacks:
@@ -699,31 +627,28 @@ if we are calling from the same thread of execution as this
     
     def _poll(self, timeout):
         callbacks = self._callbacks
+        io = self._io
         try:
-            event_pairs = self._impl.poll(timeout)
+            event_pairs = io.poll(timeout)
         except Exception as e:
             if self._raise_loop_error(e):
                 raise
-        except KeyboardInterrupt as e:
-            self.logger.warning('%s stop event loop', e.__class__.__name__)
+        except KeyboardInterrupt:
             raise StopEventLoop
         else:
-            events = []
             for fd, events in event_pairs:
-                if fd in self._handlers:
-                    callbacks.append(partial(self._handlers[fd], events))
-                else:
-                    self.logger.warning('Received an event on unregistered '\
-                                        'file descriptor %s' % fd)
+                callbacks.append(partial(io.handle_events, self, fd, events))
         
     def _call(self, callback, *args):
         try:
-            log_failure(callback(*args))
+            maybe_async(callback(*args), event_loop=self)
         except socket.error as e:
             if self._raise_loop_error(e):
-                log_failure(e, msg='Exception in event loop callback.')
+                Failure(sys.exc_info()).log(
+                    msg='Unhadled exception in event loop callback.')
         except Exception as e:
-            log_failure(e, msg='Exception in event loop callback.')
+            Failure(sys.exc_info()).log(
+                msg='Unhadled exception in event loop callback.')
 
     def _raise_loop_error(self, e):
         # Depending on python version and EventLoop implementation,
