@@ -8,7 +8,7 @@ eventually reach the recipient, provided that the recipient exists.
 
 The implementation details are outlined below:
 
-* Messages are sent via the :func:`send` function, which is a proxy for
+* Messages are sent via the :func:`pulsar.send` function, which is a proxy for
   the :class:`pulsar.Actor.send` method. Here is how you ping actor ``abc``::
       
       from pulsar import send
@@ -39,20 +39,18 @@ Actor mailbox protocol
 '''
 import sys
 import logging
-import tempfile
-from functools import partial
 from collections import namedtuple
 
-from pulsar import platform, Config, ProtocolError, CommandError
-from pulsar.utils.pep import to_bytes, ispy3k, ispy3k, pickle, set_event_loop,\
-                             new_event_loop
-from pulsar.utils.sockets import nice_address
+from pulsar import ProtocolError, CommandError
+from pulsar.utils.pep import pickle
+from pulsar.utils.internet import nice_address
 from pulsar.utils.websocket import FrameParser
 from pulsar.utils.security import gen_unique_id
 
-from .access import get_actor, set_actor
-from .defer import async, maybe_failure, log_failure, Deferred
-from .transports import ProtocolConsumer, SingleClient, Request
+from .access import get_actor
+from .defer import async, Failure, Deferred, maybe_failure, maybe_async
+from .protocols import ProtocolConsumer
+from .clients import SingleClient, Request
 from .proxy import actorid, get_proxy, get_command, ActorProxy
 
 
@@ -68,14 +66,14 @@ def command_in_context(command, caller, actor, args, kwargs):
     return cmnd(request, args, kwargs)
     
     
-class MonitorMailbox(object):
-    '''A :class:`Mailbox` for a :class:`Monitor`. This is a proxy for the
-arbiter mailbox.'''
+class ProxyMailbox(object):
+    '''A proxy for the arbiter :class:`Mailbox`.'''
     active_connections = 0
     def __init__(self, actor):
-        self.mailbox = actor.monitor.mailbox
-        # make sure the monitor get the hand shake!
-        self.mailbox.event_loop.call_soon_threadsafe(actor.hand_shake)
+        mailbox = actor.monitor.mailbox
+        if isinstance(mailbox, ProxyMailbox):
+            mailbox = mailbox.mailbox
+        self.mailbox = mailbox
 
     def __repr__(self):
         return self.mailbox.__repr__()
@@ -100,6 +98,10 @@ class Message(Request):
         self.data = data
         self.future = future
     
+    def __repr__(self):
+        return self.data.get('command', 'unknown')
+    __str__ = __repr__
+    
     @classmethod
     def command(cls, command, sender, target, args, kwargs, address=None,
                  timeout=None):
@@ -130,6 +132,9 @@ protocol.'''
         super(MailboxConsumer, self).__init__(*args, **kwargs)
         self._pending_responses = {}
         self._parser = FrameParser(kind=2)
+        actor = get_actor()
+        if actor.is_arbiter():
+            self.connection.bind_event('connection_lost', self._connection_lost)
     
     def request(self, command, sender, target, args, kwargs):
         '''Used by the server to send messages to the client.'''
@@ -145,9 +150,9 @@ protocol.'''
         while msg:
             try:
                 message = pickle.loads(msg.body)
-            except Exception:
-                raise ProtocolError('Could not decode message body')
-            log_failure(self._responde(message))
+            except Exception as e:
+                raise ProtocolError('Could not decode message body: %s' % e)
+            maybe_async(self._responde(message), event_loop=self.event_loop)
             msg = self._parser.decode()
     
     def start_request(self):
@@ -160,15 +165,22 @@ protocol.'''
                 req.future.callback(e)
         else:
             self._write(req)
-    
+            
     ############################################################################
     ##    INTERNALS
-    @async(max_errors=None)
+    def _connection_lost(self, exc):
+        actor = get_actor()
+        if not actor.is_running():
+            exc = maybe_failure(exc)
+            if isinstance(exc, Failure):
+                exc.log(msg='Connection lost with actor.', level='debug')
+    
     def _responde(self, message):
         actor = get_actor()
         command = message.get('command')
-        #LOGGER.debug('%s handling message "%s"', actor, command)
-        if command == 'callback':   #this is a callback
+        #actor.logger.debug('handling %s', command)
+        if command == 'callback':
+            # this is a callback
             self._callback(message.get('ack'), message.get('result'))
         else:
             try:
@@ -176,29 +188,27 @@ protocol.'''
                 if target is None:
                     raise CommandError('Cannot execute "%s" in %s. Unknown '
                             'actor %s' % (command, actor, message['target']))
-                caller = get_proxy(actor.get_actor(message['sender']), safe=True)
+                # Get the caller proxy without throwing
+                caller = get_proxy(actor.get_actor(message['sender']),
+                                   safe=True)
                 if isinstance(target, ActorProxy):
                     # route the message to the actor proxy
                     if caller is None:
-                        raise CommandError("'%s' got message from unknown '%s'" %
-                                           (actor, message['sender']))
+                        raise CommandError("'%s' got message from unknown '%s'"
+                                           % (actor, message['sender']))
                     result = yield actor.send(target, command, *message['args'],
                                               **message['kwargs'])
                 else:
                     actor = target
                     command = get_command(command)
                     req = CommandRequest(target, caller, self.connection)
-                    result = yield command(req, message['args'], message['kwargs'])
-            except Exception as e:
-                result = maybe_failure(e)
-            yield self._response(message, result)
-        
-    def _response(self, data, result):
-        if data.get('ack'):
-            req = Message.callback(result, data['ack'])
-            self.new_request(req)
-        #Return the result so a failure can be logged
-        return result
+                    result = yield command(req, message['args'],
+                                           message['kwargs'])
+            except Exception:
+                result = Failure(sys.exc_info())
+            if message.get('ack'):
+                req = Message.callback(result, message['ack'])
+                self.new_request(req)
 
     def _callback(self, ack, result):
         if not ack:
@@ -223,8 +233,6 @@ class MailboxClient(SingleClient):
     def __init__(self, address, actor, event_loop):
         super(MailboxClient, self).__init__(address, event_loop=event_loop)
         self.name = 'Mailbox for %s' % actor
-        # when the mailbox shutdown, the event loop must stop.
-        self.bind_event('finish', lambda s: s.event_loop.stop())
     
     def __repr__(self):
         return '%s %s' % (self.__class__.__name__, nice_address(self.address))
