@@ -47,8 +47,8 @@ Configuration
 Socket and addresses
 =======================
 
-.. automodule:: pulsar.utils.sockets
-
+.. automodule:: pulsar.utils.internet
+   :members:
 
 Internals
 =======================
@@ -83,6 +83,13 @@ HTML & Text
 .. automodule:: pulsar.utils.html
    :members:
    
+Logging
+==================
+
+.. automodule:: pulsar.utils.log
+   :members:
+   
+   
 Path
 ==================
 
