@@ -77,8 +77,10 @@ KNOWN_SETTINGS_ORDER = []
 
 
 def pass_through(arg):
-    '''A dummy function accepting on parameter only. It does nothing. It is
-used as default by :ref:`Application Hooks <setting-section-application-hooks>`.'''
+    '''A dummy function accepting one parameter only.
+    
+It does nothing and it is used as default by
+:ref:`Application Hooks <setting-section-application-hooks>`.'''
     pass
 
 def wrap_method(func):
@@ -853,24 +855,6 @@ class MaxRequests(Setting):
         restarts are disabled.
         """
 
-class Backlog(Setting):
-    name = "backlog"
-    section = "Worker Processes"
-    flags = ["--backlog"]
-    validator = validate_pos_int
-    type = int
-    default = 2048
-    desc = """\
-        The maximum number of queued connections in a socket.
-        
-        This refers to the number of clients that can be waiting to be served.
-        Exceeding this number results in the client getting an error when
-        attempting to connect. It should only affect servers under significant
-        load.
-        Must be a positive integer. Generally set in the 64-2048 range.
-        """
-
-
 class Timeout(Setting):
     name = "timeout"
     section = "Worker Processes"
@@ -883,6 +867,21 @@ class Timeout(Setting):
         killed and restarted."""
 
 
+class ThreadWorkers(Setting):
+    name = "thread_workers"
+    section = "Worker Processes"
+    flags = ["--thread-workers"]
+    validator = validate_pos_int
+    type = int
+    default = 1
+    desc = """\
+        The number of threads in an actor thread pool.
+        
+        The thread pool is used by actors to perform CPU intensive calculations.
+        In this way the actor main thread is free to listen to events on file
+        descriptors and process them as quick as possible."""
+
+
 ################################################################################
 ##    APPLICATION HOOKS
 
@@ -920,6 +919,19 @@ class WhenReady(Setting):
         This is a chance to setup :class:`pulsar.EventLoop` callbacks which
         can run periodically, at every loop or when some defined events occur.
         """
+        
+        
+class WhenExit(Setting):
+    name = "when_exit"
+    section = "Application Hooks"
+    validator = validate_callable(1)
+    type = "callable"
+    default = staticmethod(pass_through)
+    desc = """\
+        Called just before an actor is garbadge collected.
+        
+        This is a chance to check the actor status if needed.
+        """
 
 
 class ConnectionMade(Setting):
