@@ -1,11 +1,13 @@
-
+'''
+A list of all Exception specific to pulsar library.
+'''
 class PulsarException(Exception):
-    '''Base class of all Pulsar Exception'''
+    '''Base class of all Pulsar exceptions.'''
 
 
 class ImproperlyConfigured(PulsarException):
-    '''A :class:`PulsarException` raised when pulsar has inconsistent
-configuration.'''
+    '''A :class:`PulsarException` raised when an inconsistent configuration
+has occured.'''
     pass
 
 
@@ -24,43 +26,66 @@ class ProtocolError(Exception):
 the socket connection.'''
 
 
-class TooManyConnections(Exception):
+class TooManyConnections(RuntimeError):
     '''Raised when there are too many concurrent connections.'''
     
     
-class AuthenticationError(Exception):
+class EventAlreadyRegistered(RuntimeError):
     pass
-
+    
 
 class ConnectionError(Exception):
     pass
 
 
-class ActorAlreadyStarted(PulsarException):
-    '''A :class:`PulsarException` raised when trying to start an actor already started'''
-
 
+class StopEventLoop(BaseException):
+    ''':class:`BaseException` raised to cleanly stop a
+    :class:`pulsar.EventLoop`.'''
+    
+    
 class HaltServer(BaseException):
-
-    def __init__(self, reason='Halt', signal=None, exit_code=1):
-        super(HaltServer,self).__init__(reason)
+    ''':class:`BaseException` raised to stop a running server.
+    
+    When ``exit_code`` is greater than 1, it is considered an expected
+    failure and therefore the full stack trace is not logged.'''
+    def __init__(self, reason='Exiting server.', exit_code=1):
+        super(HaltServer, self).__init__(reason)
         self.exit_code = exit_code
-        self.signal = signal
 
 
 class CannotCallBackError(PulsarException):
     pass
 
 
-class AlreadyRegistered(PulsarException):
-    pass
-
 class NotRegisteredWithServer(PulsarException):
     pass
 
 
 ##################################################################### HTTP
-class HttpException(Exception):
+class HTTPError(PulsarException):
+    "Base for all HTTP related errors."
+    pass
+
+class SSLError(HTTPError):
+    "Raised when SSL certificate fails in an HTTPS connection."
+    pass
+
+class HttpException(HTTPError):
+    '''The base class of all ``HTTP`` server exceptions
+    
+Introduces the following attributes:
+
+.. attribute:: status
+
+    The numeric status code for the exception (ex 500 for server error).
+    
+    Default: ``500``.
+    
+.. attribute:: headers
+
+    Additional headers to add to the client response.
+'''
     status = 500
     def __init__(self, msg='', status=None, handler=None, strict=False,
                  headers=None, content_type=None):
@@ -78,6 +103,10 @@ class HttpException(Exception):
 
 
 class HttpRedirect(HttpException):
+    '''An :class:`HttpException` for redirects.
+    
+The :attr:`HttpException.status` is set to ``302`` by default.
+'''    
     status = 302
     def __init__(self, location, status=None, headers=None, **kw):
         headers = [] if headers is None else headers
@@ -85,15 +114,29 @@ class HttpRedirect(HttpException):
         super(HttpRedirect, self).__init__(status=status or self.status,
                                            headers=headers, **kw)
 
+    @property
+    def location(self):
+        '''The value in the ``Location`` header entry.
+        
+It is a proxy for ``self.headers['location'].'''
+        return self.headers['location']
+    
 
 class PermissionDenied(HttpException):
+    '''An :class:`HttpException` with default ``403`` status code.'''
     status = 403
 
 
 class Http404(HttpException):
+    '''An :class:`HttpException` with default ``404`` status code.'''
     status = 404
 
     
 class MethodNotAllowed(HttpException):
+    '''An :class:`HttpException` with default ``405`` status code.'''
     status = 405
-    
\ No newline at end of file
+    
+    
+class ContentNotAccepted(HttpException):
+    '''An :class:`HttpException` with default ``406`` status code.'''
+    status = 406
\ No newline at end of file
