@@ -5,16 +5,14 @@ import subprocess
 def get_version(version):
     assert len(version) == 5
     assert version[3] in ('alpha', 'beta', 'rc', 'final')
-    parts = 2 if version[2] == 0 else 3
-    main = '.'.join(map(str,version[:parts]))
+    main = '.'.join(map(str, version[:3]))
     sub = ''
     if version[3] == 'alpha' and version[4] == 0:
         git_changeset = get_git_changeset()
         if git_changeset:
-            sub = '.dev%s' % git_changeset
+            sub = '-dev.%s' % git_changeset
     elif version[3] != 'final':
-        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
-        sub = mapping[version[3]] + str(version[4])
+        sub = '-%s.%s' % tuple(version[3:])
     return main + sub
 
 def get_git_changeset():
